/* ************************************************************************
 *   File: spec_procs.c                                  Part of CircleMUD *
 *  Usage: implementation of special procedures for mobiles/objects/rooms  *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"

/*   external vars  */
extern struct time_info_data time_info;
extern struct spell_info_type spell_info[];
extern struct guild_info_type guild_info[];

/* extern functions */
ACMD(do_action);
ACMD(do_drop);
ACMD(do_flee);
ACMD(do_gen_door);
ACMD(do_say);
ACMD(do_sleep);
ACMD(do_wake);

void play_slots(struct char_data *ch);
void play_high_dice(struct char_data *ch, struct char_data *dealer, int bet);
void play_triples(struct char_data *ch, struct char_data *dealer, char *guess, int bet);
void play_seven(struct char_data *ch, struct char_data *dealer, char *guess, int bet);
void play_craps(struct char_data *ch, struct char_data *dealer, int bet);


/* local functions */
void sort_spells(void);
int compare_spells(const void *x, const void *y);
const char *how_good(int percent);
void list_skills(struct char_data *ch);
void npc_steal(struct char_data *ch, struct char_data *victim);
SPECIAL(bank);
SPECIAL(butcher);
SPECIAL(cityguard);
SPECIAL(cleric);
SPECIAL(complainer);
SPECIAL(dealer_high_dice);
SPECIAL(dealer_triples);
SPECIAL(dealer_seven);
SPECIAL(dealer_craps);
SPECIAL(dump);
SPECIAL(embalmer);
SPECIAL(fido);
SPECIAL(guild);
SPECIAL(guild_guard);
SPECIAL(janitor);
SPECIAL(magic_user);
SPECIAL(mayor);
SPECIAL(pet_shops);
SPECIAL(portal);
SPECIAL(puff);
SPECIAL(redbutton);
SPECIAL(snake);
SPECIAL(slot_machine);
SPECIAL(temple_cleric);
SPECIAL(thief);


/* ********************************************************************
 *  Special procedures for mobiles                                     *
 ******************************************************************** */

SPECIAL(dump)
{
	struct obj_data *k;
	int value = 0;

	for (k = world[IN_ROOM(ch)].contents; k; k = world[IN_ROOM(ch)].contents) {
		act("$p vanishes in a puff of smoke!", FALSE, 0, k, 0, TO_ROOM);
		extract_obj(k);
	}

	if (!CMD_IS("drop"))
		return (FALSE);

	do_drop(ch, argument, cmd, SCMD_DROP);

	for (k = world[IN_ROOM(ch)].contents; k; k = world[IN_ROOM(ch)].contents) {
		act("$p vanishes in a puff of smoke!", FALSE, 0, k, 0, TO_ROOM);
		value += MAX(1, MIN(50, GET_OBJ_COST(k) / 10));
		extract_obj(k);
	}

	if (value) {
		send_to_char(ch, "You are awarded for outstanding performance.\r\n");
		act("$n has been awarded for being a good citizen.", TRUE, ch, 0, 0, TO_ROOM);

		if (GET_LEVEL(ch) < 3)
			gain_exp(ch, value);
		else
			GET_COPPER(ch) += value;
	}
	return (TRUE);
}


SPECIAL(mayor)
{
	char actbuf[MAX_INPUT_LENGTH];

	const char open_path[] =
		"W3a3003b33000c111d0d111Oe333333Oe22c222112212111a1S.";
	const char close_path[] =
		"W3a3003b33000c111d0d111CE333333CE22c222112212111a1S.";

	static const char *path = NULL;
	static int path_index;
	static bool move = FALSE;

	if (!move) {
		if (time_info.hours == 6) {
			move = TRUE;
			path = open_path;
			path_index = 0;
		} else if (time_info.hours == 20) {
			move = TRUE;
			path = close_path;
			path_index = 0;
		}
	}
	if (cmd || !move || (GET_POS(ch) < POS_SLEEPING) ||
		 (GET_POS(ch) == POS_FIGHTING))
		return (FALSE);

	switch (path[path_index]) {
		case '0':
		case '1':
		case '2':
		case '3':
			perform_move(ch, path[path_index] - '0', 1);
			break;

		case 'W':
			GET_POS(ch) = POS_STANDING;
			act("$n awakens and groans loudly.", FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'S':
			GET_POS(ch) = POS_SLEEPING;
			act("$n lies down and instantly falls asleep.", FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'a':
			act("$n says 'Hello Honey!'", FALSE, ch, 0, 0, TO_ROOM);
			act("$n smirks.", FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'b':
			act("$n says 'What a view!  I must get something done about that dump!'",
				 FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'c':
			act("$n says 'Vandals!  Youngsters nowadays have no respect for anything!'",
				 FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'd':
			act("$n says 'Good day, citizens!'", FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'e':
			act("$n says 'I hereby declare the bazaar open!'", FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'E':
			act("$n says 'I hereby declare Midgaard closed!'", FALSE, ch, 0, 0, TO_ROOM);
			break;

		case 'O':
			do_gen_door(ch, strcpy(actbuf, "gate"), 0, SCMD_UNLOCK);	/* strcpy: OK */
			do_gen_door(ch, strcpy(actbuf, "gate"), 0, SCMD_OPEN);	/* strcpy: OK */
			break;

		case 'C':
			do_gen_door(ch, strcpy(actbuf, "gate"), 0, SCMD_CLOSE);	/* strcpy: OK */
			do_gen_door(ch, strcpy(actbuf, "gate"), 0, SCMD_LOCK);	/* strcpy: OK */
			break;

		case '.':
			move = FALSE;
			break;

	}

	path_index++;
	return (FALSE);
}


/* ********************************************************************
 *  General special procedures for mobiles                             *
 ******************************************************************** */


void npc_steal(struct char_data *ch, struct char_data *victim)
{
	int gold;

	if (IS_NPC(victim))
		return;
	if (GET_LEVEL(victim) >= LVL_IMMORT)
		return;
	if (!CAN_SEE(ch, victim))
		return;

	if (AWAKE(victim) && (rand_number(0, GET_LEVEL(ch)) == 0)) {
		act("You discover that $n has $s hands in your wallet.", 
			 FALSE, ch, 0, victim, TO_VICT);
		act("$n tries to steal gold from $N.", 
			 TRUE, ch, 0, victim, TO_NOTVICT);
	} else {
		/* Steal some gold coins */

		if (GET_GOLD(victim)) {
			gold = (GET_GOLD(victim) * rand_number(1, 10)) / 100;
			if (gold > 0) {
				GET_GOLD(ch) += gold;
				GET_GOLD(victim) -= gold;
			}
		} else if (GET_PLATINUM(victim)) { 
			gold = (GET_PLATINUM(victim) * rand_number(1, 10)) / 100;
			if (gold > 0) {
				GET_PLATINUM(ch) += gold;
				GET_PLATINUM(victim) -= gold;
			}
		} else if (GET_SILVER(victim)) {
			gold = (GET_SILVER(victim) * rand_number(1, 10)) / 100;
			if (gold > 0) {
				GET_SILVER(ch) += gold;
				GET_SILVER(victim) -= gold;
			}
		} else {
			gold = (GET_COPPER(victim) * rand_number(1, 10)) / 100;
			if (gold > 0) {
				GET_COPPER(ch) += gold;
				GET_COPPER(victim) -= gold;
			}
		}
	}
}


/*
 * Quite lethal to low-level characters.
 */
SPECIAL(snake)
{
	if (cmd || GET_POS(ch) != POS_FIGHTING || !FIGHTING(ch))
		return (FALSE);

	if (IN_ROOM(FIGHTING(ch)) != IN_ROOM(ch) || rand_number(0, GET_LEVEL(ch)) != 0)
		return (FALSE);

	act("$n bites $N!", 1, ch, 0, FIGHTING(ch), TO_NOTVICT);
	act("$n bites you!", 1, ch, 0, FIGHTING(ch), TO_VICT);
	call_magic(ch, FIGHTING(ch), 0, SPELL_POISON, GET_LEVEL(ch), CAST_SPELL, NULL);
	return (TRUE);
}


SPECIAL(thief)
{
	struct char_data *cons;

	if (cmd || GET_POS(ch) != POS_STANDING)
		return (FALSE);

	for (cons = world[IN_ROOM(ch)].people; cons; cons = cons->next_in_room)
		if (!IS_NPC(cons) && GET_LEVEL(cons) < LVL_IMMORT && !rand_number(0, 4)) {
			npc_steal(ch, cons);
			return (TRUE);
		}

	return (FALSE);
}


SPECIAL(magic_user)
{
	struct char_data *vict;

	if (cmd || GET_POS(ch) != POS_FIGHTING)
		return (FALSE);

	/* pseudo-randomly choose someone in the room who is fighting me */
	for (vict = world[IN_ROOM(ch)].people; vict; vict = vict->next_in_room)
		if (FIGHTING(vict) == ch && !rand_number(0, 4))
			break;

	/* if I didn't pick any of those, then just slam the guy I'm fighting */
	if (vict == NULL && IN_ROOM(FIGHTING(ch)) == IN_ROOM(ch))
		vict = FIGHTING(ch);

	/* Hm...didn't pick anyone...I'll wait a round. */
	if (vict == NULL)
		return (TRUE);

	if (GET_LEVEL(ch) > 13 && rand_number(0, 10) == 0)
		cast_spell(ch, vict, NULL, SPELL_POISON, NULL);

	if (GET_LEVEL(ch) > 7 && rand_number(0, 8) == 0)
		cast_spell(ch, vict, NULL, SPELL_BLINDNESS, NULL);

	if (GET_LEVEL(ch) > 12 && rand_number(0, 12) == 0) {
		if (IS_EVIL(ch))
			cast_spell(ch, vict, NULL, SPELL_ENERGY_DRAIN, NULL);
		else if (IS_GOOD(ch))
			cast_spell(ch, vict, NULL, SPELL_DISPEL_EVIL, NULL);
	}

	if (rand_number(0, 4))
		return (TRUE);

	switch (GET_LEVEL(ch)) {
		case 4:
		case 5:
			cast_spell(ch, vict, NULL, SPELL_MAGIC_MISSILE, NULL);
			break;
		case 6:
		case 7:
			cast_spell(ch, vict, NULL, SPELL_CHILL_TOUCH, NULL);
			break;
		case 8:
		case 9:
			cast_spell(ch, vict, NULL, SPELL_BURNING_HANDS, NULL);
			break;
		case 10:
		case 11:
			cast_spell(ch, vict, NULL, SPELL_SHOCKING_GRASP, NULL);
			break;
		case 12:
		case 13:
			cast_spell(ch, vict, NULL, SPELL_LIGHTNING_BOLT, NULL);
			break;
		case 14:
		case 15:
		case 16:
		case 17:
			cast_spell(ch, vict, NULL, SPELL_COLOR_SPRAY, NULL);
			break;
		default:
			cast_spell(ch, vict, NULL, SPELL_FIREBALL, NULL);
			break;
	}
	return (TRUE);

}


/* ********************************************************************
 *  Special procedures for mobiles                                      *
 ******************************************************************** */

SPECIAL(guild_guard)
{
	int i;
	struct char_data *guard = (struct char_data *)me;
	const char *buf = "The guard humiliates you, and blocks your way.\r\n";
	const char *buf2 = "The guard humiliates $n, and blocks $s way.";

	if (!IS_MOVE(cmd) || AFF_FLAGGED(guard, AFF_BLIND))
		return (FALSE);

	if (GET_LEVEL(ch) >= LVL_IMMORT)
		return (FALSE);

	for (i = 0; guild_info[i].guild_room != NOWHERE; i++) {
		/* Wrong guild or not trying to enter. */
		if (GET_ROOM_VNUM(IN_ROOM(ch)) != guild_info[i].guild_room || cmd != guild_info[i].direction)
			continue;

		/* Allow the people of the guild through. */
		if (!IS_NPC(ch) && GET_CLASS(ch) == guild_info[i].pc_class)
			continue;

		send_to_char(ch, "%s", buf);
		act(buf2, FALSE, ch, 0, 0, TO_ROOM);
		return (TRUE);
	}

	return (FALSE);
}



SPECIAL(puff)
{
	char actbuf[MAX_INPUT_LENGTH];

	if (cmd)
		return (FALSE);

	switch (rand_number(0, 60)) {
		case 0:
			do_say(ch, strcpy(actbuf, "My god!  It's full of stars!"), 0, 0);	/* strcpy: OK */
			return (TRUE);
		case 1:
			do_say(ch, strcpy(actbuf, "How'd all those fish get up here?"), 0, 0);	/* strcpy: OK */
			return (TRUE);
		case 2:
			do_say(ch, strcpy(actbuf, "I'm a very female dragon."), 0, 0);	/* strcpy: OK */
			return (TRUE);
		case 3:
			do_say(ch, strcpy(actbuf, "I've got a peaceful, easy feeling."), 0, 0);	/* strcpy: OK */
			return (TRUE);
		default:
			return (FALSE);
	}
}



SPECIAL(fido)
{
	struct obj_data *i, *temp, *next_obj;

	if (cmd || !AWAKE(ch))
		return (FALSE);

	for (i = world[IN_ROOM(ch)].contents; i; i = i->next_content) {
		if (!IS_CORPSE(i))
			continue;

		act("$n savagely devours a corpse.", FALSE, ch, 0, 0, TO_ROOM);
		for (temp = i->contains; temp; temp = next_obj) {
			next_obj = temp->next_content;
			obj_from_obj(temp);
			obj_to_room(temp, IN_ROOM(ch));
		}
		extract_obj(i);
		return (TRUE);
	}

	return (FALSE);
}



SPECIAL(janitor)
{
	struct obj_data *i;

	if (cmd || !AWAKE(ch))
		return (FALSE);

	for (i = world[IN_ROOM(ch)].contents; i; i = i->next_content) {
		if (!CAN_WEAR(i, ITEM_WEAR_TAKE))
			continue;
		if (GET_OBJ_TYPE(i) != ITEM_DRINKCON && GET_OBJ_COST(i) >= 15)
			continue;
		act("$n picks up some trash.", FALSE, ch, 0, 0, TO_ROOM);
		obj_from_room(i);
		obj_to_char(i, ch);
		return (TRUE);
	}

	return (FALSE);
}


SPECIAL(cityguard)
{
	struct char_data *tch, *evil, *spittle;
	int max_evil, min_cha;

	if (cmd || !AWAKE(ch) || FIGHTING(ch))
		return (FALSE);

	max_evil = 1000;
	min_cha = 6;
	spittle = evil = NULL;

	for (tch = world[IN_ROOM(ch)].people; tch; tch = tch->next_in_room) {
		if (!CAN_SEE(ch, tch))
			continue;

		if (!IS_NPC(tch) && PLR_FLAGGED(tch, PLR_KILLER)) {
			act("$n screams 'HEY!!!  You're one of those PLAYER KILLERS!!!!!!'", FALSE, ch, 0, 0, TO_ROOM);
			hit(ch, tch, TYPE_UNDEFINED);
			return (TRUE);
		}

		if (!IS_NPC(tch) && PLR_FLAGGED(tch, PLR_THIEF)) {
			act("$n screams 'HEY!!!  You're one of those PLAYER THIEVES!!!!!!'", FALSE, ch, 0, 0, TO_ROOM);
			hit(ch, tch, TYPE_UNDEFINED);
			return (TRUE);
		}

		if (FIGHTING(tch) && GET_ALIGNMENT(tch) < max_evil && (IS_NPC(tch) || IS_NPC(FIGHTING(tch)))) {
			max_evil = GET_ALIGNMENT(tch);
			evil = tch;
		}

		if (GET_CHA(tch) < min_cha) {
			spittle = tch;
			min_cha = GET_CHA(tch);
		}
	}

	if (evil && GET_ALIGNMENT(FIGHTING(evil)) >= 0) {
		act("$n screams 'PROTECT THE INNOCENT!  BANZAI!  CHARGE!  ARARARAGGGHH!'",
			 FALSE, ch, 0, 0, TO_ROOM);
		hit(ch, evil, TYPE_UNDEFINED);
		return (TRUE);
	}

	/* Reward the socially inept. */
	if (spittle && !rand_number(0, 9)) {
		static int spit_social;

		if (!spit_social)
			spit_social = find_command("spit");

		if (spit_social > 0) {
			char spitbuf[MAX_NAME_LENGTH + 1];

			strncpy(spitbuf, GET_NAME(spittle), sizeof(spitbuf));	/* strncpy: OK */
			spitbuf[sizeof(spitbuf) - 1] = '\0';

			do_action(ch, spitbuf, spit_social, 0);
			return (TRUE);
		}
	}

	return (FALSE);
}


#define PET_PRICE(pet) (GET_LEVEL(pet) * 300)

SPECIAL(pet_shops)
{
	char buf[MAX_STRING_LENGTH], pet_name[256];
	room_rnum pet_room;
	struct char_data *pet;

	/* Gross. */
	pet_room = IN_ROOM(ch) + 1;

	if (CMD_IS("list")) {
		send_to_char(ch, "Available pets are:\r\n");
		for (pet = world[pet_room].people; pet; pet = pet->next_in_room) {
			/* No, you can't have the Implementor as a pet if he's in there. */
			if (!IS_NPC(pet))
				continue;
			send_to_char(ch, "%8d - %s\r\n", PET_PRICE(pet), GET_NAME(pet));
		}
		return (TRUE);
	} else if (CMD_IS("buy")) {

		two_arguments(argument, buf, pet_name);

		if (!(pet = get_char_room(buf, NULL, pet_room)) || !IS_NPC(pet)) {
			send_to_char(ch, "There is no such pet!\r\n");
			return (TRUE);
		}
		if (convert_all_to_copper(ch) < PET_PRICE(pet)) {
			send_to_char(ch, "You don't have enough gold!\r\n");
			return (TRUE);
		}
		make_change(ch, PET_PRICE(pet), BUYING);

		pet = read_mobile(GET_MOB_RNUM(pet), REAL);
		GET_EXP(pet) = 0;
		AFF_FLAGS(pet).AFF_CHARM = 1;

		if (*pet_name) {
			snprintf(buf, sizeof(buf), "%s %s", pet->player.name, pet_name);
			/* free(pet->player.name); don't free the prototype! */
			pet->player.name = strdup(buf);

			snprintf(buf, sizeof(buf), "%sA small sign on a chain around the neck says 'My name is %s'\r\n",
						pet->player.description, pet_name);
			/* free(pet->player.description); don't free the prototype! */
			pet->player.description = strdup(buf);
		}
		char_to_room(pet, IN_ROOM(ch));
		add_follower(pet, ch);

		/* Be certain that pets can't get/carry/use/wield/wear items */
		IS_CARRYING_W(pet) = 1000;
		IS_CARRYING_N(pet) = 100;

		send_to_char(ch, "May you enjoy your pet.\r\n");
		act("$n buys $N as a pet.", FALSE, ch, 0, pet, TO_ROOM);

		return (TRUE);
	}

	/* All commands except list and buy */
	return (FALSE);
}



/* ********************************************************************
 *  Special procedures for objects                                     *
 ******************************************************************** */


SPECIAL(bank)
{
	int amount, type;
	char bal[MAX_STRING_LENGTH];
	char arg[MAX_INPUT_LENGTH], currency[MAX_INPUT_LENGTH];

	if (CMD_IS("balance")) {
		if (GET_BANK_GOLD(ch) > 0) {
			send_to_char(ch, "Your current balance is: ");
			send_to_char(ch, list_from_copper(ch, GET_BANK_GOLD(ch), bal, UNCONDENSED_LIST));
		} else
			send_to_char(ch, "You currently have no money deposited.\r\n");
		return (TRUE);
	} else if (CMD_IS("deposit")) {
		argument = one_argument(argument, arg);
		if (!*arg || !is_number(arg)) {
			send_to_char(ch, "How much do you want to deposit?\r\n");
			return (TRUE);
		}

		amount = atoi(arg);
		one_argument(argument, currency);
		if (!strcmp(currency, "platinum") || is_abbrev(currency, "platinum"))
			type = PLAT_COINS;
		else if (!strcmp(currency, "gold") || is_abbrev(currency, "gold"))
			type = GOLD_COINS;
		else if (!strcmp(currency, "silver") || is_abbrev(currency, "silver"))
			type = SILVER_COINS;
		else if (!strcmp(currency, "copper") || is_abbrev(currency, "copper"))
			type = COPPER_COINS;
		else {
			send_to_char(ch, "Deposit what kind of coins?\r\n");
			return (FALSE);
		}

		if (GET_MONEY(ch, type) < amount) {
			send_to_char(ch, "You don't have that many coins!\r\n");
			return (TRUE);
		}
		GET_MONEY(ch, type) -= amount;
		GET_BANK_GOLD(ch) += convert_to_copper(amount, type);
		send_to_char(ch, "You deposit %d coins.\r\n", amount);
		act("$n makes a bank transaction.", TRUE, ch, 0, FALSE, TO_ROOM);
		return (TRUE);
	} else if (CMD_IS("withdraw")) {
		argument = one_argument(argument, arg);
		if (!*arg || !is_number(arg)) {
			send_to_char(ch, "How much do you want to withdraw?\r\n");
			return (TRUE);
		}

		amount = atoi(arg);
		one_argument(argument, currency);
		if (!strcmp(currency, "platinum") || is_abbrev(currency, "platinum"))
			type = PLAT_COINS;
		else if (!strcmp(currency, "gold") || is_abbrev(currency, "gold"))
			type = GOLD_COINS;
		else if (!strcmp(currency, "silver") || is_abbrev(currency, "silver"))
			type = SILVER_COINS;
		else if (!strcmp(currency, "copper") || is_abbrev(currency, "copper"))
			type = COPPER_COINS;
		else {
			send_to_char(ch, "Withdraw what kind of coins?\r\n");
			return (TRUE);
		}

		if (GET_BANK_GOLD(ch) < convert_to_copper(amount, type)) {
			send_to_char(ch, "You don't have that many coins deposited!\r\n");
			return (TRUE);
		}
		add_money_to_char(ch, amount, type);
		GET_BANK_GOLD(ch) -= convert_to_copper(amount, type);

		send_to_char(ch, "You withdraw %d %s coins.\r\n", 
						 amount, coin_types[type]);
		act("$n makes a bank transaction.", TRUE, ch, 0, FALSE, TO_ROOM);
		return (TRUE);
	}
	return (FALSE);
}


/*
  From: Mike <claywar@cetlink.net>
  Subject: 8-Ball Special Procedure!

  Feel like a useless specproc to liven your day?  Try a low feature 8ball!
*/
SPECIAL(ball)
{
	char obj_name[MAX_STRING_LENGTH];

	if (CMD_IS("rub")) {
		argument = one_argument(argument, obj_name);
		if (!(get_obj_in_list_vis(ch, obj_name, NULL, ch->carrying)))
			return (FALSE);

		switch (dice(1, 5)) {
			case 1:
				send_to_char(ch, "Yes.\r\n");
				break;
			case 2:
				send_to_char(ch, "Maybe.\r\n");
				break;
			case 3:
				send_to_char(ch, "No.\r\n");
				break;
			case 4:
				send_to_char(ch, "Uncertain.\r\n");
				break;
			case 5:
				send_to_char(ch, "Ask again later.\r\n");
				break;
			default:
				send_to_char(ch, "BUG!!! Please Report!\r\n");
				break;
		}
		return (TRUE);
	}
	return (FALSE);
}


/*
  From: Necromonitor <necro@praxis.annex.net>
  Subject: Temple Clerics

  This is a spec_proc for a temple cleric.
  He'll cure poison first, then blindness, and THEN heal people of
  their wounds.  He heals the person who has the least percentage
  of their maxhit first.  Developed, and in use on ImmortalQuest/2.
  telnet to annex.net, port 2001

  Developed by Necro.
*/

SPECIAL(temple_cleric)
{
	struct char_data *vict;
	struct char_data *hitme = NULL;
	static int this_hour;
	float temp1 = 1;
	float temp2 = 1;

	if (cmd)
		return (FALSE);

	if (time_info.hours != 0) {
		this_hour = time_info.hours;

		for (vict = world[IN_ROOM(ch)].people; vict; vict = vict->next_in_room) {
			if (AFF_FLAGGED(vict,AFF_POISON))
				hitme = vict;
      }

		if (hitme != NULL) {
			cast_spell(ch, hitme, NULL, SPELL_REMOVE_POISON, NULL);
			return (TRUE);
      }

		for (vict = world[IN_ROOM(ch)].people; vict; vict = vict->next_in_room)
			if (AFF_FLAGGED(vict, AFF_BLIND))
				hitme = vict;

		if (hitme != NULL) {
			cast_spell(ch, hitme, NULL, SPELL_CURE_BLIND, NULL);
			return (TRUE);
      }

		for (vict = world[IN_ROOM(ch)].people; vict; vict = vict->next_in_room) {
			temp1 = (float) GET_HIT(vict) / (float) GET_MAX_HIT(vict);
			if (temp1 < temp2) {
				temp2 = temp1;
				hitme = vict;
			}
      }

		if (hitme != NULL) {
			cast_spell(ch, hitme, NULL, SPELL_HEAL, NULL);
			return (TRUE);
      }
	}
	return (FALSE);
}


/* This special procedure makes a mob into a 'rent-a-cleric', who sells spells
   by the sea shore... uuh, maybe not.  Anyway, the mob will also cast certain
   spells on low-level characters in the room for free.  
   By:  Wyatt Bode	Date:  April, 1996
*/
SPECIAL(cleric)
{
	int i;
	char buf[MAX_STRING_LENGTH];
	struct char_data *vict;
	struct char_data *cleric = (struct char_data *)me;
	struct price_info {
		short int number;
		char name[25];
		short int price;
	} prices[] = {
		/* Spell Num (defined)	Name shown	  Price  */
		{ SPELL_ARMOR, 		"armor          ", 100 },
		{ SPELL_BLESS, 		"bless          ", 200 },
		{ SPELL_REMOVE_POISON, 	"remove poison  ", 500 },
		{ SPELL_CURE_BLIND, 	"cure blindness ", 500 },
		{ SPELL_CURE_CRITIC, "critic         ", 700 },
		{ SPELL_SANCTUARY, 	"sanctuary      ", 1500 },
		{ SPELL_HEAL, 		   "heal           ", 2000 },

		/* The next line must be last, add new spells above. */ 
		{ -1, "\r\n", -1 }
	};

/* NOTE:  In interpreter.c, you must define a command called 'heal' for this
   spec_proc to work.  Just define it as do_not_here, and the mob will take 
   care of the rest.  (If you don't know what this means, look in interpreter.c
   for a clue.)
*/

	if (CMD_IS("heal")) {
		argument = one_argument(argument, buf);

		if (GET_POS(ch) == POS_FIGHTING) 
			return TRUE;

		if (*buf) {
			for (i = 0; prices[i].number > SPELL_RESERVED_DBC; i++) {
				if (is_abbrev(buf, prices[i].name)) 
					if (convert_all_to_copper(ch) < prices[i].price) {
						act("$n tells you, 'You don't have enough gold for that "
							 "spell!'", FALSE, cleric, NULL, ch, TO_VICT);
						return (TRUE);
					} else {
						act("$N gives $n some money.", 
							 FALSE, cleric, 0, ch, TO_NOTVICT);
						send_to_char(ch, "You give %s %d coins.\r\n", 
										 GET_NAME(cleric), prices[i].price);
						make_change(ch, prices[i].price, BUYING);

						cast_spell(cleric, ch, NULL, prices[i].number, NULL);
						return (TRUE);
          
					}
			}
			act("$n tells you, 'I do not know of that spell!"
				 "  Type 'heal' for a list.'", FALSE, cleric, 0, ch, TO_VICT);
	  
			return (TRUE);
		} else {
			act("$n tells you, 'Here is a listing of the prices for my "
				 "services.'", FALSE, cleric, 0, ch, TO_VICT);
			for (i = 0; prices[i].number > SPELL_RESERVED_DBC; i++) {
				send_to_char(ch, "%s%d\r\n", prices[i].name, prices[i].price);
			}
			return (TRUE);
		}
	}

	if (cmd) 
		return (FALSE);

	/* pseudo-randomly choose someone in the room */
	for (vict = world[IN_ROOM(ch)].people; vict; vict = vict->next_in_room)
		if (!rand_number(0, 3))
			break;
  
	/* change the level at the end of the next line to control free spells */
	if (vict == NULL || IS_NPC(vict) || (GET_LEVEL(vict) > 10))
		return (FALSE);

	switch (rand_number(1, GET_LEVEL(vict))) { 
      case 1: cast_spell(ch, vict, NULL, SPELL_CURE_LIGHT, NULL); break; 
      case 2: cast_spell(ch, vict, NULL, SPELL_BLESS, NULL); break; 
      case 3: cast_spell(ch, vict, NULL, SPELL_ARMOR, NULL); break;
      case 4: cast_spell(ch, vict, NULL, SPELL_CURE_LIGHT, NULL); break; 
      case 5: cast_spell(ch, vict, NULL, SPELL_BLESS, NULL); break; 
      case 6: cast_spell(ch, vict, NULL, SPELL_CURE_CRITIC, NULL); break; 
      case 7: cast_spell(ch, vict, NULL, SPELL_ARMOR, NULL); break;
      case 8: cast_spell(ch, vict, NULL, SPELL_CURE_CRITIC, NULL); break; 
      case 9: cast_spell(ch, vict, NULL, SPELL_ARMOR, NULL); break; 
      case 10: 
			/* special wacky thing, your mileage may vary */ 
			act("$n utters the words, 'energizer'.", TRUE, ch, 0, vict, TO_ROOM);
			act("You feel invigorated!", FALSE, ch, 0, vict, TO_VICT);
			GET_MANA(vict) = MIN(GET_MAX_MANA(vict), MAX((GET_MANA(vict) + 10), rand_number(50, 200)));
			break; 
	}
	return (TRUE); 
}               


int get_people_room(const int room)
{
	struct char_data *person;
	int i = 0;

	for (person = world[room].people; person; person = person->next_in_room) {
      if (person)
			i++;
	}

	return i;
}

struct char_data *select_person_room(const int room, const int whom)
{
	struct char_data *person;
	int i = get_people_room(room);

	if (i == 0)
		return NULL;

	if (whom > i)
		return NULL;

	if (whom == 1)
      return (world[room].people);

	i = whom;
	i -= 1;

	for (person = world[room].people; i; person = person->next_in_room)
      i -= 1;

	return person;
}

SPECIAL(complainer)
{
	struct char_data *vict1, *self = (struct char_data *)me;

	if (!AWAKE(self) && !AFF_FLAGGED(self, AFF_SLEEP)) {
      if (cmd) {
			if (rand_number(0, 1)) {
				act("$n awakens.", TRUE, self, 0, 0, TO_ROOM);
				GET_POS(self) = POS_STANDING;
			}
			return (0);
		} else {
			if (rand_number(1, 5) == 5) {
				act("$n awakens.", TRUE, self, 0, 0, TO_ROOM);
				GET_POS(self) = POS_STANDING;
			}
			return (1);
		}
	}

	if (CMD_IS("flee") && CAN_SEE(self, ch)) {
      switch (rand_number(1, 6)) {
			case 1:
            act("$n tells you 'Run! Run! Flee while you can!'", 
					 FALSE, self, 0, ch, TO_VICT);
				break;
			case 2:
            act("$n tells you 'You wimp.'",
					 FALSE, self, 0, ch, TO_VICT);
				break;
			case 3:
            act("$n tells you 'You wuss! haha...'", 
					 FALSE, self, 0, ch, TO_VICT);
				break;
			default:
				break;
		}

      do_flee(ch, argument, cmd, 0);

      switch (rand_number(1, 6)) {
			case 1:
            act("$n says 'I bet $N just peed $S pants.'", 
					 FALSE, self, 0, ch, TO_ROOM);
				break;
			case 2:
            act("$n says 'I heard $N runs from $S own shadow too.'", 
					 FALSE, self, 0, ch, TO_ROOM);
				break;
			case 3:
            act("$n says 'I bet $N's afraid of the dark too.'", 
					 FALSE, self, 0, ch, TO_ROOM);
				break;
			default:
				break;
		}
      return (1);
	}

	if (CMD_IS("sleep") && CAN_SEE(self, ch)) {
      do_sleep(ch, argument, cmd, 0);

      switch (rand_number(1, 6)) {
			case 1:
            do_say(self, "Ok everyone... Shhhhhh.", 0, 0);
				break;

			case 2:
            do_say(self, "That's a good idea... I'm going to sleep too.", 0, 0);
            do_sleep(self,0,0,0);
				break;

			case 3:
            do_say(self, "Heh. What a sleepyhead.", 0, 0);
				break;

			default:
				break;
		}
      return (1);
	}

	if (cmd)
		return (0);

	if (FIGHTING(self)) {
      switch (rand_number(1, 10)) {
			case 1:
            do_say(self,"I think I'm losing...",0,0);
				break;
			case 2:
            do_say(self,"The world won't miss a loudmouth like me. Hah.",0,0);
				break;
			case 3:
            if(GET_HIT(self) < (GET_MAX_HIT(self)/3)) {
					do_say(self,"I'll live...",0,0);
					do_say(self,"My cuts are only flesh wounds.",0,0);
				}
				break;
			default:
				break;
		}
      return (1);
	}

	if (get_people_room(IN_ROOM(self)) <= 1)
		return (0);

	vict1 = select_person_room(IN_ROOM(self),
										rand_number(1, get_people_room(IN_ROOM(self))));
	
	if (vict1 == self)
		return (0);

	if (!CAN_SEE(self, vict1))
		return (0);

	if (FIGHTING(vict1)) {
      if (rand_number(0, 1) && CAN_SEE(self, vict1) && FIGHTING(vict1) != self) {
			vict1 = FIGHTING(vict1);  /* swap the person we're talking about */
			/* makes it more interesting. */
		}

      switch(rand_number(1, 20)) {
			case 1:
            act("$n says '$N started it!'", 
					 FALSE, self, 0, vict1, TO_NOTVICT);
            act("$n says 'I KNOW you started it!'", 
					 FALSE, self, 0, vict1, TO_VICT);
				break;
			case 2:
            act("$n says '$N's fighting technique is so lame...'", 
					 FALSE, self, 0, vict1, TO_NOTVICT);
            act("$n says 'Your fighting technique is so lame...'", 
					 FALSE, self, 0, vict1, TO_VICT);
				break;
			case 3:
            act("$n says 'Hehe. I think $N is going to die.'", 
					 FALSE, self, 0, vict1,TO_NOTVICT);
				break;
			case 4:
            act("$n says 'Hehe. I hope $N loses. $E's such a jerk.'", 
					 FALSE, self, 0, vict1, TO_NOTVICT);
			default:
				break;
		}
      return (1);
	}

	/* just a summary of how we can get to this point :P (for those who
		might be lost already): execution gets here only if the mob
		assigned with this is not sleeping, not fighting, if a person
		hasn't fled or gone to sleep, or the person we've chosen isn't
		fighting and awake. */

	switch (rand_number(1, 40)) {
      case 1:
			do_say(self,"I talk too much!",0,0);
			break;
      case 2:
			do_say(self,"I think I'm brainless.",0,0);
			break;
      case 3:
			do_say(self,"I think YOU'RE brainless.",0,0);
			break;
      case 4:
			do_say(self,"People are stupid.",0,0);
			break;
      case 5:
			do_say(self,"Who's to blame?",0,0);
			break;
      case 6:
			if (IS_NPC(vict1) && MEMORY(vict1))
				act("$n says 'I think $N is after someone.'", 
					 FALSE, self, 0, vict1, TO_NOTVICT);
			break;
      case 7:
			act("$n says 'I think $N sleeps too much.'", 
				 FALSE, self, 0, vict1, TO_NOTVICT);
			break;
      case 8:
			act("$n says 'Run everyone!!! $N is going to kill you!'", 
				 FALSE, self, 0, vict1, TO_NOTVICT);
			break;
      case 9:
			act("$n says 'I saw $N eat the crumbs on the floor of the"
				 " bakery.'", FALSE, self, 0, vict1, TO_NOTVICT);
			act("$n says 'Bleeech.'", FALSE, self, 0, vict1, TO_ROOM);
			break;
      case 10:
			act("$n says 'I think $N did it. I'm sure...'", 
				 FALSE, self, 0, vict1, TO_NOTVICT);
			break;
      default:
			break;
	}
	return (1);
}

SPECIAL (portal)
{
	struct obj_data *obj = (struct obj_data *)me;
	struct obj_data *port;
	char obj_name[MAX_STRING_LENGTH];

	if (!CMD_IS("enter")) 
		return (FALSE);

	argument = one_argument(argument, obj_name);
	if (!(port = get_obj_in_list_vis(ch, obj_name, NULL, 
												world[IN_ROOM(ch)].contents)))
      return(FALSE);
    
	if (port != obj)
      return (FALSE);
    
	if (port->obj_flags.value[1] <= 0 ||
		 port->obj_flags.value[1] > top_of_world) {
      send_to_char(ch, "The portal leads nowhere\n\r");
      return (TRUE);
	}
    
	act("$n enters $p, and vanishes!", 
		 FALSE, ch, port, 0, TO_ROOM);
	act("You enter $p, and you are transported elsewhere", 
		 FALSE, ch, port, 0, TO_CHAR);

	char_from_room(ch);  
	char_to_room(ch, port->obj_flags.value[1]);
	look_at_room(ch,0);

	act("$n appears from thin air!", FALSE, ch, 0, 0, TO_ROOM);

	return (TRUE);
}


#define NUM_BODY_PARTS 5

static char *headers[] =
{
	"the corpse of The ",
	"the corpse of the ",
	"the corpse of an ",
	"the corpse of An ",
	"the corpse of a ",
	"the corpse of A ",
	"the corpse of "
};

char *body_part[]=
{
	"an arm of",
	"a leg of",
	"a foot of",
	"a rib of",
	"a hand of",
	"\n"
};

char *body_word[]=
{
	"arm",
	"leg",
	"foot",
	"rib",
	"hand",
	"\n"
};


SPECIAL(butcher)
{
	struct obj_data *obj;
	char loc_buf[MAX_STRING_LENGTH]="";
	char loc_buf2[MAX_STRING_LENGTH]="";
	char buf[MAX_INPUT_LENGTH];
	int i = 0, len = 0, total_len = 0, done = FALSE, part = 0;

	if (!cmd && !FIGHTING((struct char_data *)me)) {
      if (rand_number(1, 50) < 2) {
			act("$n says 'Anyone have a corpse for me? I can butcher them "
				 "for food!'", FALSE, me, 0, 0, TO_ROOM);
			return (TRUE);
		}
      return (FALSE);
	}

	if (CMD_IS("butcher")) {
      argument = one_argument(argument, buf);
      if (strcmp(buf, "corpse") == 0) {
			obj = get_obj_in_list_vis(ch, "corpse", NULL, ch->carrying);
			if(!obj) {
				act("$N says to you 'Sorry. You don't seem to have"
					 " any corpses.'", FALSE, ch, 0, me, TO_CHAR);
				return (TRUE);
			}

			if (convert_all_to_copper(ch) < 500) {
				act("$N says to you 'Sorry. You don't have"
					 " enough gold.'", FALSE, ch, 0, me, TO_CHAR);
				return (TRUE);
			} else {
				part = rand_number(1, NUM_BODY_PARTS) - 1;

				for (i = 0; (i < 7) || (!done); i++) {
					len = strlen(headers[i]);
					if(memcmp(obj->short_description, headers[i], len) == 0) {
						total_len = strlen(obj->short_description);
						strncpy(loc_buf, obj->short_description+len, total_len-len);

						free(obj->name);
						snprintf(loc_buf2, sizeof(loc_buf2),
									"%s %s", body_word[part], loc_buf);
						obj->name = strdup(loc_buf2);

						snprintf(loc_buf2, sizeof(loc_buf2), 
									"%s %s lies here.", body_part[part], loc_buf);
						free(obj->description);
						obj->description = strdup(loc_buf2);

						snprintf(loc_buf2, sizeof(loc_buf2), 
									"%s %s", body_part[part], loc_buf);
						free(obj->short_description);
						obj->short_description = strdup(loc_buf2);

						GET_OBJ_TYPE(obj)   = ITEM_FOOD;
						GET_OBJ_WEAR(obj).ITEM_WEAR_TAKE = 1;
						GET_OBJ_VAL(obj,0)  = 5; /* adjust this for fill */
						GET_OBJ_VAL(obj,1)  = 0;
						GET_OBJ_VAL(obj,2)  = 0;
						GET_OBJ_VAL(obj,3)  = 0; /* adjust this for poison */
						GET_OBJ_WEIGHT(obj) = 5; /* adjust this for weight */
						GET_OBJ_COST(obj)   = 500;
						GET_OBJ_RENT(obj)   = 100; /* shouldn't be rentable,
																unless you make a change to
																your mud to do so - corpses
																have a vnum & rnum of -1. */
						GET_OBJ_TIMER(obj)  = -1;

						make_change(ch, 1000, BUYING);
						done = TRUE;

						act("$N says to you 'Done! Enjoy your meal!'",
							 FALSE, ch, 0, me, TO_CHAR);
						return (TRUE);
					}
				}

				act("$N says to you 'Sorry, I can't make cuts with"
					 "this.'", FALSE, ch, 0, me, TO_CHAR);
				return (TRUE);
			}
		}
      return (FALSE); /* if it wasn't corpse... it wasn't meant for me... */
	}
	return (FALSE);
}


SPECIAL(embalmer)
{
	struct obj_data *obj;
	char buf[MAX_STRING_LENGTH], loc_buf[MAX_STRING_LENGTH] = "";
	char loc_buf2[MAX_STRING_LENGTH] = "";
	int i = 0, len = 0, total_len = 0, done = FALSE;

	if (!cmd && !FIGHTING((struct char_data *)me)) {
      if (rand_number(1, 50) < 2) {
			act("$n says 'Anyone have a corpse for me? I can make them "
				 "into bags!'", FALSE, me, 0, 0, TO_ROOM);
			return (TRUE);
		}
      return (FALSE);
	}

	if (CMD_IS("embalm")) {
      argument = one_argument(argument, buf);

		if (strcmp(buf, "corpse") == 0) {
			obj = get_obj_in_list_vis(ch, "corpse", NULL, ch->carrying);
			if (!obj) {
				act("$N says to you 'Sorry. You don't seem to have any corpses.'",
					 FALSE, ch, 0, me, TO_CHAR);
				return (TRUE);
			}

			if (convert_all_to_copper(ch) < 1000) {
				act("$N says to you 'Sorry. You don't have enough gold.'", 
					 FALSE, ch, 0, me, TO_CHAR);
				return (TRUE);
			} else {
				for (i = 0; (i < 7) || (!done); i++) {
					len = strlen(headers[i]);

					if (memcmp(obj->short_description,headers[i],len) == 0) {
						total_len = strlen(obj->short_description);
						strncpy(loc_buf, 
								  obj->short_description + len,
								  total_len - len);
						free(obj->name);
						snprintf(loc_buf2, sizeof(loc_buf2),
									"bag skin %s", loc_buf);
						obj->name = strdup(loc_buf2);

						snprintf(loc_buf2, sizeof(loc_buf2),
									"A bag of %s skin lies here", loc_buf);
						free(obj->description);
						obj->description = strdup(loc_buf2);

						snprintf(loc_buf2, sizeof(loc_buf2),
									"a bag made from %s skin", loc_buf);
						free(obj->short_description);
						obj->short_description = strdup(loc_buf2);

						GET_OBJ_TYPE(obj)   = ITEM_CONTAINER;
						GET_OBJ_WEAR(obj).ITEM_WEAR_TAKE = 1;
						GET_OBJ_VAL(obj, 0) = 100; /* adjust this for capacity */
						GET_OBJ_WEIGHT(obj) = 5;  /* adjust this for weight */
						GET_OBJ_COST(obj)   = 1000;
						GET_OBJ_RENT(obj)   = 200;  /* shouldn't be
																 rentable..  unless
																 you make a change to
																 your mud to do so,
																 because corpses have
																 a vnum&rnum of -1 */
						GET_OBJ_TIMER(obj) = -1;
						make_change(ch, 1000, BUYING);
						done = TRUE;
						act("$N says to you 'Done! Enjoy your new bag.'",
							 FALSE, ch, 0, me, TO_CHAR);
						return (TRUE);
					}
				}
				act("$N says to you 'Sorry, I can't make a bag from this.'", 
					 FALSE, ch, 0, me, TO_CHAR);
				return (TRUE);
			}
		}
      return (FALSE); /* if it wasn't corpse... it wasn't meant for me... */
	}
	return (FALSE);
}


SPECIAL (redbutton)
{
	struct obj_data *obj = (struct obj_data *) me;
	struct obj_data *port;
	char obj_name[MAX_STRING_LENGTH];

	if (!CMD_IS("push")) return FALSE;

	argument = one_argument(argument,obj_name);
	if (!(port = get_obj_in_list_vis(ch, obj_name,NULL, world[IN_ROOM(ch)].contents)))
      return(FALSE);

	if (port != obj)
      return(FALSE);

	act("$n pushes $p and fades away.", FALSE, ch, port, 0, TO_ROOM);
	act("You push $p, and you are transported elsewhere",
		 FALSE, ch, port, 0, TO_CHAR);

	char_from_room(ch);
	char_to_room(ch, rand_number(0, top_of_world));
	look_at_room(ch, 0);

	act("$n slowly materializes from nowhere...", FALSE, ch, 0, 0, TO_ROOM);

	return(TRUE);
}


/* weapon_spell() function and special procedures -dak */
void weapon_spell(char *to_ch, char *to_vict, char *to_room,
						struct char_data *ch, struct char_data *vict, 
						struct obj_data *obj, int spl)
{
   int level = LVL_IMPL+1, i;
   
   for (i = 0; i < NUM_CLASSES; i++)
		if (spell_info[spl].min_level[i] < level)
			level = spell_info[spl].min_level[i];

   level = MAX(1, MIN(LVL_IMMORT-1, level));
 
   act(to_ch, FALSE, ch, obj, vict, TO_CHAR);
   act(to_vict, FALSE, ch, obj, vict, TO_VICT);
   act(to_room, FALSE, ch, obj, vict, TO_NOTVICT);
   call_magic(ch, vict, 0, spl, level, CAST_SPELL, 0);
}
 
 
SPECIAL(blind_weapon)
{
   struct char_data *vict = FIGHTING(ch);
   
   if (cmd || !vict || !rand_number(0, 9))
		return (FALSE);
   if (AFF_FLAGGED(vict, AFF_BLIND) || MOB_FLAGGED(vict, MOB_NOBLIND))
		return (FALSE);
     
   weapon_spell("You scream, \"MIDNIGHT!\" at $N.",
                "$n screams, \"Midnight!\" at you.",
                "$n screams, \"Midnight!\" at $N.",
                ch, vict, (struct obj_data *) me, SPELL_BLINDNESS);
   return (TRUE);
}
 

SPECIAL(fireball_weapon)
{
   struct char_data *vict = FIGHTING(ch);
   
   if (cmd || !vict || !rand_number(0, 14))
		return (FALSE);
     
   weapon_spell("Fire seems to shoot from your $o and roar through the air at $N!",
                "Fire tears from $n's $o and roars through the air at you!",
                "$n's $o comes alive with a fireball that roars towards $N.",
                ch, vict, (struct obj_data *) me, SPELL_FIREBALL);
   return (TRUE);
}

 
SPECIAL(curse_weapon)
{
   struct char_data *vict = FIGHTING(ch);
   
   if (cmd || !vict || !rand_number(0, 11))
		return (FALSE);
   if (AFF_FLAGGED(vict, AFF_CURSE))
		return (FALSE);
     
   weapon_spell("Your $o turns black for a brief moment.",
                "$n's $o turns obsidian as it nears contact with you.",
                "$n's $o turns black as it contacts $N.",
                ch, vict, (struct obj_data *) me, SPELL_CURSE);
   return (TRUE);
}


SPECIAL(slot_machine)
{
	if (!CMD_IS("pull"))
		return (FALSE);
	
	play_slots(ch);

	return (TRUE);
}


SPECIAL(dealer_high_dice)
{
	struct char_data *dealer = (struct char_data *)me;
	char buf[MAX_INPUT_LENGTH];
	int bet;

	if (!CMD_IS("dice"))
		return (FALSE);
	
	argument = one_argument(argument, buf);
	if (!*buf) {
		send_to_char(ch, "The dealer tells you: dice <bet>\r\n");
		return (FALSE);
	}
	
	bet = atoi(buf);
	if (bet <= 0) {
		send_to_char(ch, "The dealer tells you: You must bet some money!\r\n");
		return (FALSE);
	}

	play_high_dice(ch, dealer, bet);
	
	return (TRUE);
}


SPECIAL(dealer_craps)
{
	struct char_data *dealer = (struct char_data *)me;
	char buf[MAX_INPUT_LENGTH];
	int bet;

	if (!CMD_IS("dice"))
		return (FALSE);
	
	argument = one_argument(argument, buf);
	if (!*buf) {
		send_to_char(ch, "The dealer tells you: dice <bet>\r\n");
		return (FALSE);
	}
	
	bet = atoi(buf);
	if (bet <= 0) {
		send_to_char(ch, "The dealer tells you: You must bet some money!\r\n");
		return (FALSE);
	}

	play_craps(ch, dealer, bet);
	
	return (TRUE);
}


SPECIAL(dealer_triples)
{
	struct char_data *dealer = (struct char_data *)me;
	char guess[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
	int bet;

	if (!CMD_IS("dice"))
		return (FALSE);
	
	argument = two_arguments(argument, guess, buf);
	if (!*buf || !*guess) {
		send_to_char(ch, "The dealer tells you: dice <guess> <bet>\r\n");
		return (FALSE);
	}
	
	bet = atoi(buf);
	if (bet <= 0) {
		send_to_char(ch, "The dealer tells you: You must bet some money!\r\n");
		return (FALSE);
	}

	play_triples(ch, dealer, guess, bet);
	
	return (TRUE);
}


SPECIAL(dealer_seven)
{
	struct char_data *dealer = (struct char_data *)me;
	char guess[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
	int bet;

	if (!CMD_IS("dice"))
		return (FALSE);
	
	argument = two_arguments(argument, guess, buf);
	if (!*buf || !*guess) {
		send_to_char(ch, "The dealer tells you: dice <guess> <bet>\r\n");
		return (FALSE);
	}
	
	bet = atoi(buf);
	if (bet <= 0) {
		send_to_char(ch, "The dealer tells you: You must bet some money!\r\n");
		return (FALSE);
	}

	play_seven(ch, dealer, guess, bet);
	
	return (TRUE);
}
