/* ************************************************************************
 *   File: spells.c                                      Part of CircleMUD *
 *  Usage: Implementation of "manual spells".  Circle 2.2 spell compat.    *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */


#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"


/* external variables */
extern room_rnum top_of_world;
extern room_rnum r_mortal_start_room;
extern int mini_mud;
extern int pk_allowed;

/* external functions */
ACMD(do_flee);
void clearMemory(struct char_data *ch);
void weight_change_object(struct obj_data *obj, int weight);
int mag_savingthrow(struct char_data *ch, int type, int modifier);
void name_to_drinkcon(struct obj_data *obj, int type);
void name_from_drinkcon(struct obj_data *obj);
int compute_armor_class(struct char_data *ch);
void force_weather_change(int change, zone_rnum zone); /* weather.c */

/*
 * Special spells appear below.
 */

ASPELL(spell_create_water)
{
	int water;

	if (ch == NULL || obj == NULL)
		return;
	/* level = MAX(MIN(level, LVL_IMPL), 1);	 - not used */

	if (GET_OBJ_TYPE(obj) == ITEM_DRINKCON) {
		if ((GET_OBJ_VAL(obj, 2) != LIQ_WATER) && (GET_OBJ_VAL(obj, 1) != 0)) {
			name_from_drinkcon(obj);
			GET_OBJ_VAL(obj, 2) = LIQ_SLIME;
			name_to_drinkcon(obj, LIQ_SLIME);
		} else {
			water = MAX(GET_OBJ_VAL(obj, 0) - GET_OBJ_VAL(obj, 1), 0);
			if (water > 0) {
				if (GET_OBJ_VAL(obj, 1) >= 0)
					name_from_drinkcon(obj);
				GET_OBJ_VAL(obj, 2) = LIQ_WATER;
				GET_OBJ_VAL(obj, 1) += water;
				name_to_drinkcon(obj, LIQ_WATER);
				weight_change_object(obj, water);
				act("$p is filled.", FALSE, ch, obj, 0, TO_CHAR);
			}
		}
	}
}


int can_recall(struct char_data *ch)
{
	if (ch == NULL || IS_NPC(ch))
		return (FALSE);

	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_NO_RECALL)) {
		send_to_char(ch, "A magical force prevents recall!\r\n");
		return (FALSE);
	}

	if (ZONE_FLAGGED(IN_ZONE(ch), ZONE_NORECALL)) {
		send_to_char(ch, "A mystical force stops you.\r\n");
		return (FALSE);
	}

	return (TRUE);
}


ASPELL(spell_recall)
{
	if (!can_recall(victim))
		return;
	
	act("$n disappears.", TRUE, victim, 0, 0, TO_ROOM);
	char_from_room(victim);
	char_to_room(victim, r_mortal_start_room);
	act("$n appears in the middle of the room.", TRUE, victim, 0, 0, TO_ROOM);
	look_at_room(victim, 0);
}


/*
  From: Franco <awe@mt.arias.net>
  Subject: [Circle] New Teleport

  I thought the relocate spell was overused, so I rewrote the teleport spell
  (how often is the original used anyway, cept by a few snoopy morts?). You
  have to change the spello/mag_assign_spell or wotever functions in
  'spell_parser.c' to have the spell accept a TAR_VICT_WORLD (I think).
*/

/* New and improved teleport spell */
ASPELL(spell_teleport)
{
	struct room_data *newroom = NULL;
	struct room_data *casting_room, *target_room;
	struct obj_data *foolz_objs;
	int randroom = 0, prob = 0, NR = 0, i;

	if (ch == NULL || victim == NULL)
		return;

	casting_room = &world[IN_ROOM(ch)];
	target_room  = &world[IN_ROOM(victim)];

	if (GET_LEVEL(victim) > LVL_IMMORT && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your mortal magic is foiled.\r\n");
		return;
	}

	if (casting_room->room_flags.ROOM_NOTELEPORT) {
		send_to_char(ch, "This room blocks your attempt to teleport.\r\n");
		return;
	}

	if (target_room->room_flags.ROOM_NOTELEPORT) {
		send_to_char(ch, "The destination room blocks your teleport.\r\n");
		return;
	}

	if (casting_room->room_flags.ROOM_NOMAGIC) {
		send_to_char(ch, "Your magic fizzles out and dies.\r\n");
		return;
	}

	if (IS_NPC(victim)) {
		send_to_char(ch, "You cannot get an image of the location..\r\n");
		return;
	}

	newroom = &world[IN_ROOM(victim)];

	if (newroom->room_flags.ROOM_NOMAGIC) {
		send_to_char(ch, "Your destination's anti-magic barrier prevents the "
						 "spell.\r\n");
		return;
	}

	if (newroom->room_flags.ROOM_TUNNEL) {
		send_to_char(ch, "The size of the destination prevents your "
						 "teleportation.\r\n");
		return;
	}

	prob = rand_number(1,100);

	if (GET_LEVEL(ch) >= LVL_IMMORT)
		prob = 50;

	if (prob >= 3 && prob <= 90) {
		send_to_char(ch, "Your magic takes affect as your begin to shift "
						 "locations...\r\n");
		act("$n slowly disappears, teleporting away.", 
			 TRUE, ch, 0, 0, TO_NOTVICT);

		NR = IN_ROOM(victim);
		char_from_room(ch);
		char_to_room(ch, NR);

		act("$n teleports into the room.", TRUE, ch, 0, 0, TO_ROOM);
		send_to_char(ch, "You have teleported on target.\r\n");
		look_at_room(ch, 0);
		return;
	}

	if (prob < 3) {
		send_to_char(ch, "Your spell goes awry as you teleport into "
						 "solid matter!\r\n");

		act("$n miscasts a teleport spell, and disappears.", 
			 TRUE, ch, 0, 0, TO_NOTVICT);

		for (i = 0; i < NUM_WEARS; i++)
			if (GET_EQ(ch, i))
				unequip_char(ch, i);

		while (ch->carrying) {
			foolz_objs = ch->carrying;
			extract_obj(foolz_objs);
		}

		extract_char(ch);
		mudlog(BRF, LVL_IMMORT, TRUE, "(GC) %s has misteleported, attempting to "
				 "teleport to %s, and died.", GET_NAME(ch), GET_NAME(victim));
		return;
	}

	if (prob < 90) {
		send_to_char(ch, "Your spell goes awry as you teleport into an "
						 "unknown place!\r\n");
		act("$n miscasts a teleport spell, and disappears.", 
			 TRUE, 0, 0, 0, TO_NOTVICT);

      do {
			randroom = rand_number(0, top_of_world);
      } while (ROOM_FLAGGED(randroom, ROOM_PRIVATE) || 
					ROOM_FLAGGED(randroom, ROOM_DEATH) ||
					ROOM_FLAGGED(randroom, ROOM_GODROOM));
		
		char_from_room(ch);
		char_to_room(ch, randroom);
		act("$n teleports into the room.", TRUE, ch, 0, 0, TO_ROOM);
		send_to_char(ch, "You luckily have not died during teleportation.\r\n");
		look_at_room(victim, 0);
		return;
	}
}


#if 0
/* The original teleport spell that nobody ever uses: */
ASPELL(spell_teleport)
{
	room_rnum to_room;

	if (victim == NULL || IS_NPC(victim))
		return;

	do {
		to_room = rand_number(0, top_of_world);
	} while (ROOM_FLAGGED(to_room, ROOM_PRIVATE) ||
				ROOM_FLAGGED(to_room, ROOM_DEATH) ||
				ROOM_FLAGGED(to_room, ROOM_GODROOM));

	act("$n slowly fades out of existence and is gone.",
		 FALSE, victim, 0, 0, TO_ROOM);
	char_from_room(victim);
	char_to_room(victim, to_room);
	act("$n slowly fades into existence.", FALSE, victim, 0, 0, TO_ROOM);
	look_at_room(victim, 0);
}
#endif


/* Based on Franco's teleport spell, it just removes the check for
 * miscasting */
ASPELL(spell_relocate)
{
	struct room_data *newroom = NULL;
	struct room_data *casting_room, *target_room;

	if (ch == NULL || victim == NULL)
		return;

	casting_room = &world[IN_ROOM(ch)];
	target_room  = &world[IN_ROOM(victim)];

	if (GET_LEVEL(victim) > LVL_IMMORT && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your mortal magic is foiled.\r\n");
		return;
	}

	if (casting_room->room_flags.ROOM_NOTELEPORT) {
		send_to_char(ch, "This room blocks your attempt to teleport.\r\n");
		return;
	}

	if (target_room->room_flags.ROOM_NOTELEPORT) {
		send_to_char(ch, "The destination room blocks your teleport.\r\n");
		return;
	}

	if (casting_room->room_flags.ROOM_NOMAGIC) {
		send_to_char(ch, "Your magic fizzles out and dies.\r\n");
		return;
	}

	if (IS_NPC(victim)) {
		send_to_char(ch, "You cannot get an image of the location..\r\n");
		return;
	}

	newroom = &world[IN_ROOM(victim)];

	if (newroom->room_flags.ROOM_NOMAGIC) {
		send_to_char(ch, "Your destination's anti-magic barrier prevents the "
						 "spell.\r\n");
		return;
	}

	if (newroom->room_flags.ROOM_TUNNEL) {
		send_to_char(ch, "The size of the destination prevents your "
						 "teleportation.\r\n");
		return;
	}

	send_to_char(ch, "Your magic takes affect as your begin to shift "
					 "locations...\r\n");
	act("$n slowly disappears, teleporting away.", 
		 TRUE, ch, 0, 0, TO_NOTVICT);

	char_from_room(ch);
	char_to_room(ch, IN_ROOM(victim));

	act("$n teleports into the room.", TRUE, ch, 0, 0, TO_ROOM);
	send_to_char(ch, "You have teleported on target.\r\n");
	look_at_room(ch, 0);
	return;

}



#define SUMMON_FAIL "You failed.\r\n"

ASPELL(spell_summon)
{
	if (ch == NULL || victim == NULL)
		return;

	if (GET_LEVEL(victim) > MIN(LVL_IMMORT - 1, level + 3)) {
		send_to_char(ch, SUMMON_FAIL);
		return;
	}

	if (ZONE_FLAGGED(IN_ZONE(ch), ZONE_NOSUMMON)) {
		send_to_char(ch, "Your magic fails.\r\n");
		return;
	}

	if (!pk_allowed) {
		if (MOB_FLAGGED(victim, MOB_AGGRESSIVE)) {
			act("As the words escape your lips and $N travels\r\n"
				 "through time and space towards you, you realize that $E is\r\n"
				 "aggressive and might harm you, so you wisely send $M back.",
				 FALSE, ch, 0, victim, TO_CHAR);
			return;
		}
		if (!IS_NPC(victim) && !PRF_FLAGGED(victim, PRF_SUMMONABLE) &&
			 !PLR_FLAGGED(victim, PLR_KILLER)) {
			send_to_char(victim, "%s just tried to summon you to: %s.\r\n"
							 "%s failed because you have summon protection on.\r\n"
							 "Type NOSUMMON to allow other players to summon you.\r\n",
							 GET_NAME(ch), world[IN_ROOM(ch)].name,
							 (ch->player.sex == SEX_MALE) ? "He" : "She");

			send_to_char(ch, "You failed because %s has summon protection on.\r\n",
							 GET_NAME(victim));
			mudlog(BRF, LVL_IMMORT, TRUE, "%s failed summoning %s to %s.", 
					 GET_NAME(ch), GET_NAME(victim), world[IN_ROOM(ch)].name);
			return;
		}
	}

	if (MOB_FLAGGED(victim, MOB_NOSUMMON) ||
		 (IS_NPC(victim) && mag_savingthrow(victim, SAVING_SPELL, 0))) {
		send_to_char(ch, SUMMON_FAIL);
		return;
	}

	act("$n disappears suddenly.", TRUE, victim, 0, 0, TO_ROOM);

	char_from_room(victim);
	char_to_room(victim, IN_ROOM(ch));

	act("$n arrives suddenly.", TRUE, victim, 0, 0, TO_ROOM);
	act("$n has summoned you!", FALSE, ch, 0, victim, TO_VICT);
	look_at_room(victim, 0);
}



ASPELL(spell_locate_object)
{
	struct obj_data *i;
	char name[MAX_INPUT_LENGTH];
	int j;

	/* use the temporary object that do_cast created */
	if (!tar_str) {
		send_to_char(ch, "What object would you like to find?\r\n");
		return;
	}
	strcpy(name, tar_str);

	j = level / 2;

	for (i = object_list; i && (j > 0); i = i->next) {
		if (!isname(name, i->name))
			continue;

		send_to_char(ch, "%c%s", UPPER(*i->short_description), i->short_description);

		if (CARRIED_BY(i))
			send_to_char(ch, " is being carried by %s.\r\n", PERS(CARRIED_BY(i), ch));
		else if (IN_ROOM(i) != NOWHERE)
			send_to_char(ch, " is in %s.\r\n", world[IN_ROOM(i)].name);
		else if (IN_OBJ(i))
			send_to_char(ch, " is in %s.\r\n", IN_OBJ(i)->short_description);
		else if (WORN_BY(i))
			send_to_char(ch, " is being worn by %s.\r\n", PERS(WORN_BY(i), ch));
		else
			send_to_char(ch, "'s location is uncertain.\r\n");

		j--;
	}

	if (j == level / 2)
		send_to_char(ch, "You sense nothing.\r\n");
}



ASPELL(spell_charm)
{
	struct affected_type af;

	if (victim == NULL || ch == NULL)
		return;

	if (victim == ch)
		send_to_char(ch, "You like yourself even better!\r\n");
	else if (!IS_NPC(victim) && !PRF_FLAGGED(victim, PRF_SUMMONABLE))
		send_to_char(ch, "You fail because SUMMON protection is on!\r\n");
	else if (AFF_FLAGGED(victim, AFF_SANCTUARY))
		send_to_char(ch, "Your victim is protected by sanctuary!\r\n");
	else if (MOB_FLAGGED(victim, MOB_NOCHARM))
		send_to_char(ch, "Your victim resists!\r\n");
	else if (AFF_FLAGGED(ch, AFF_CHARM))
		send_to_char(ch, "You can't have any followers of your own!\r\n");
	else if (AFF_FLAGGED(victim, AFF_CHARM) || level < GET_LEVEL(victim))
		send_to_char(ch, "You fail.\r\n");
	/* player charming another player - no legal reason for this */
	else if (!pk_allowed && !IS_NPC(victim))
		send_to_char(ch, "You fail - shouldn't be doing it anyway.\r\n");
	else if (IS_ELF(victim) && rand_number(1, 100) <= 90)
		send_to_char(ch, "Your victim resists!\r\n");
	else if (IS_ELF(victim) && rand_number(1, 100) <= 90)
		send_to_char(ch, "Your victim resists!\r\n");
	else if (circle_follow(victim, ch))
		send_to_char(ch, "Sorry, following in circles can not be allowed.\r\n");
	else if (mag_savingthrow(victim, SAVING_PARA, 0))
		send_to_char(ch, "Your victim resists!\r\n");
	else {
		if (victim->master)
			stop_follower(victim);

		add_follower(victim, ch);

		af.type = SPELL_CHARM;
		af.duration = 24 * 2;
		if (GET_CHA(ch))
			af.duration *= GET_CHA(ch);
		if (GET_INT(victim))
			af.duration /= GET_INT(victim);
		af.modifier = 0;
		af.location = 0;
		af.bitvector.AFF_CHARM = 1;
		affect_to_char(victim, &af);

		act("Isn't $n just such a nice fellow?", FALSE, ch, 0, victim, TO_VICT);
		if (IS_NPC(victim))
			MOB_FLAGS(victim).MOB_SPEC = 0;
	}
}



ASPELL(spell_identify)
{
	int i, found;
	size_t len;

	if (obj) {
		char bitbuf[MAX_STRING_LENGTH];

		sprinttype(GET_OBJ_TYPE(obj), item_types, bitbuf, sizeof(bitbuf));
		send_to_char(ch, "You feel informed:\r\nObject '%s', Item type: %s\r\n", obj->short_description, bitbuf);

		if (!ISEMPTY(GET_OBJ_AFFECT(obj))) {
			sprintfield((char *)&obj->obj_flags.bitvector, affected_bits, 
							bitbuf, sizeof(bitbuf));
			send_to_char(ch, "Item will give you following abilities:  %s\r\n", 
							 bitbuf);
		}

		sprintfield((char *)&GET_OBJ_EXTRA(obj), extra_bits, 
						bitbuf, sizeof(bitbuf));
		send_to_char(ch, "Item is: %s\r\n", bitbuf);

		send_to_char(ch, "Weight: %d, Value: %d, Rent: %d\r\n", 
						 GET_OBJ_WEIGHT(obj), GET_OBJ_COST(obj), GET_OBJ_RENT(obj));

		switch (GET_OBJ_TYPE(obj)) {
			case ITEM_SCROLL:
			case ITEM_POTION:
				len = i = 0;

				if (GET_OBJ_VAL(obj, 1) >= 1) {
					i = snprintf(bitbuf + len, sizeof(bitbuf) - len, " %s", skill_name(GET_OBJ_VAL(obj, 1)));
					if (i >= 0)
						len += i;
				}

				if (GET_OBJ_VAL(obj, 2) >= 1 && len < sizeof(bitbuf)) {
					i = snprintf(bitbuf + len, sizeof(bitbuf) - len, " %s", skill_name(GET_OBJ_VAL(obj, 2)));
					if (i >= 0)
						len += i;
				}

				if (GET_OBJ_VAL(obj, 3) >= 1 && len < sizeof(bitbuf)) {
					i = snprintf(bitbuf + len, sizeof(bitbuf) - len, " %s", skill_name(GET_OBJ_VAL(obj, 3)));
					if (i >= 0)
						len += i;
				}

				send_to_char(ch, "This %s casts: %s\r\n", item_types[(int) GET_OBJ_TYPE(obj)], bitbuf);
				break;
			case ITEM_WAND:
			case ITEM_STAFF:
				send_to_char(ch, "This %s casts: %s\r\nIt has %d maximum charge%s and %d remaining.\r\n",
								 item_types[(int) GET_OBJ_TYPE(obj)], skill_name(GET_OBJ_VAL(obj, 3)),
								 GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 1) == 1 ? "" : "s", GET_OBJ_VAL(obj, 2));
				break;
			case ITEM_WEAPON:
				send_to_char(ch, "Damage Dice is '%dD%d' for an average per-round damage of %.1f.\r\n",
								 GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2), ((GET_OBJ_VAL(obj, 2) + 1) / 2.0) * GET_OBJ_VAL(obj, 1));
				break;
			case ITEM_ARMOR:
				send_to_char(ch, "AC-apply is %d\r\n", GET_OBJ_VAL(obj, 0));
				break;
		}
		found = FALSE;
		for (i = 0; i < MAX_OBJ_AFFECT; i++) {
			if ((obj->affected[i].location != APPLY_NONE) &&
				 (obj->affected[i].modifier != 0)) {
				if (!found) {
					send_to_char(ch, "Can affect you as :\r\n");
					found = TRUE;
				}
				sprinttype(obj->affected[i].location, apply_types, bitbuf, sizeof(bitbuf));
				send_to_char(ch, "   Affects: %s By %d\r\n", bitbuf, obj->affected[i].modifier);
			}
		}
	} else if (victim) {		/* victim */
		send_to_char(ch, "Name: %s\r\n", GET_NAME(victim));
		if (!IS_NPC(victim))
			send_to_char(ch, "%s is %d years, %d months, %d days and %d hours old.\r\n",
							 GET_NAME(victim), age(victim)->year, age(victim)->month,
							 age(victim)->day, age(victim)->hours);
		send_to_char(ch, "Height %d cm, Weight %d pounds\r\n", GET_HEIGHT(victim), GET_WEIGHT(victim));
		send_to_char(ch, "Level: %d, Hits: %d, Mana: %d\r\n", GET_LEVEL(victim), GET_HIT(victim), GET_MANA(victim));
		send_to_char(ch, "AC: %d, Hitroll: %d, Damroll: %d\r\n", compute_armor_class(victim), GET_HITROLL(victim), GET_DAMROLL(victim));
		send_to_char(ch, "Str: %d/%d, Int: %d, Wis: %d, Dex: %d, Con: %d, Cha: %d\r\n",
						 GET_STR(victim), GET_ADD(victim), GET_INT(victim),
						 GET_WIS(victim), GET_DEX(victim), GET_CON(victim), GET_CHA(victim));
	}
}



/*
 * Cannot use this spell on an equipped object or it will mess up the
 * wielding character's hit/dam totals.
 */
ASPELL(spell_enchant_weapon)
{
	int i, flag;

	if (ch == NULL || obj == NULL)
		return;

	/* Either already enchanted or not a weapon. */
	if (GET_OBJ_TYPE(obj) != ITEM_WEAPON || OBJ_FLAGGED(obj, ITEM_MAGIC))
		return;

	/* Make sure no other affections. */
	for (i = flag = 0; i+1 < MAX_OBJ_AFFECT && !flag; i++) {
      if (obj->affected[i].location == APPLY_NONE)
			flag = 1;
	}

	GET_OBJ_EXTRA(obj).ITEM_MAGIC = 1;
	GET_OBJ_EXTRA(obj).ITEM_UNIQUE_SAVE = 1;

	obj->affected[i].location = APPLY_HITROLL;
	obj->affected[i].modifier = 1 + (level >= 18);

	obj->affected[i+1].location = APPLY_DAMROLL;
	obj->affected[i+1].modifier = 1 + (level >= 20);

	if (IS_GOOD(ch)) {
		GET_OBJ_EXTRA(obj).ITEM_ANTI_EVIL = 1;
		act("$p glows blue.", FALSE, ch, obj, 0, TO_CHAR);
	} else if (IS_EVIL(ch)) {
		GET_OBJ_EXTRA(obj).ITEM_ANTI_GOOD = 1;
		act("$p glows red.", FALSE, ch, obj, 0, TO_CHAR);
	} else
		act("$p glows yellow.", FALSE, ch, obj, 0, TO_CHAR);
}


ASPELL(spell_detect_poison)
{
	if (victim) {
		if (victim == ch) {
			if (AFF_FLAGGED(victim, AFF_POISON))
				send_to_char(ch, "You can sense poison in your blood.\r\n");
			else
				send_to_char(ch, "You feel healthy.\r\n");
		} else {
			if (AFF_FLAGGED(victim, AFF_POISON))
				act("You sense that $E is poisoned.", FALSE, ch, 0, victim, TO_CHAR);
			else
				act("You sense that $E is healthy.", FALSE, ch, 0, victim, TO_CHAR);
		}
	}

	if (obj) {
		switch (GET_OBJ_TYPE(obj)) {
			case ITEM_DRINKCON:
			case ITEM_FOUNTAIN:
			case ITEM_FOOD:
				if (GET_OBJ_VAL(obj, 3))
					act("You sense that $p has been contaminated.",FALSE,ch,obj,0,TO_CHAR);
				else
					act("You sense that $p is safe for consumption.", FALSE, ch, obj, 0,
						 TO_CHAR);
				break;
			default:
				send_to_char(ch, "You sense that it should not be consumed.\r\n");
		}
	}
}


ASPELL(spell_fear)
{
	struct char_data *target = (struct char_data *) victim;
	struct char_data *next_target;
	char buf[MAX_STRING_LENGTH];
	int rooms_to_flee = 0;

	if (ch == NULL)
		return;

	send_to_char(ch, "You radiate an aura of fear into the room!\r\n");
	act("$n is surrounded by an aura of fear!", TRUE, ch, 0, 0, TO_ROOM);

	for (target = world[IN_ROOM(ch)].people; target; target = next_target) {
		next_target = target->next_in_room;

		if (target == NULL)
			return;

		if (target == ch || GET_LEVEL(target) >= LVL_IMMORT)
			continue;

		if (mag_savingthrow(target, SAVING_SPELL, 0)) {
			snprintf(buf, sizeof(buf), "%s is unaffected by the fear!\r\n",
						GET_NAME(target));
			act(buf, TRUE, ch, 0, 0, TO_ROOM);
			send_to_char(ch, "Your victim is not afraid of the likes of you!\r\n");
			if (IS_NPC(target))
				hit(target, ch, TYPE_UNDEFINED);

		} else {
			for (rooms_to_flee = level / 10; rooms_to_flee > 0; rooms_to_flee--) {
				send_to_char(target, "You flee in terror!\r\n");
				do_flee(target, "", 0, 0);
			}
		}
	}
}


#define PORTAL 31

ASPELL(spell_portal)
{
	/* Check target room for legality. */
	struct room_data *nrp, *rp = &world[IN_ROOM(ch)];
	struct obj_data *tmp_obj2, *tmp_obj = read_object(PORTAL, VIRTUAL);
	struct char_data *tmp_ch = (struct char_data *) victim;
	struct extra_descr_data *ed;
	char buf[512], str[180];

	if (!rp || !tmp_obj) {
		send_to_char(ch, "The magic fails\r\n");
		extract_obj(tmp_obj);
		return;
	}

	if (rp->room_flags.ROOM_NOMAGIC) {
		send_to_char(ch, "Eldritch wizardry obstructs thee.\r\n");
		extract_obj(tmp_obj);
		return;
	}

	if (rp->room_flags.ROOM_TUNNEL) {
		send_to_char(ch, "There is no room in here to summon!\r\n");
		extract_obj(tmp_obj);
		return;
	}

	if (!(nrp = &world[IN_ROOM(tmp_ch)])) {
		snprintf(str, sizeof(str), "%s not in any room", GET_NAME(tmp_ch));
		log(str);
		send_to_char(ch, "The magic cannot locate the target\r\n");
		extract_obj(tmp_obj);
		return;
	}

	if (ROOM_FLAGGED(IN_ROOM(tmp_ch), ROOM_NOMAGIC)) {
		send_to_char(ch, "Your target is protected against your magic.\r\n");
		extract_obj(tmp_obj);
		return;
	}

	snprintf(buf, sizeof(buf),
				"Through the mists of the portal, you can faintly see %s", 
				nrp->name);

	CREATE(ed , struct extra_descr_data, 1);
	ed->next = tmp_obj->ex_description;
	tmp_obj->ex_description = ed;
	CREATE(ed->keyword, char, strlen(tmp_obj->name) + 1);
	strcpy(ed->keyword, tmp_obj->name);
	ed->description = strdup(buf);

	tmp_obj->obj_flags.value[0] = level/5;
	tmp_obj->obj_flags.value[1] = IN_ROOM(tmp_ch);

	obj_to_room(tmp_obj, IN_ROOM(ch));

	act("$p suddenly appears.", TRUE, ch, tmp_obj, 0, TO_ROOM);
	act("$p suddenly appears.", TRUE, ch, tmp_obj, 0, TO_CHAR);

	/* Portal at other side */
   rp = &world[IN_ROOM(ch)];
   tmp_obj2 = read_object(PORTAL, VIRTUAL);
   if (!rp || !tmp_obj2) {
		send_to_char(ch, "The magic fails\r\n");
		extract_obj(tmp_obj2);
		return;
   }

	snprintf(buf, sizeof(buf),
				"Through the mists of the portal, you can faintly see %s", 
				rp->name);

	CREATE(ed , struct extra_descr_data, 1);
	ed->next = tmp_obj2->ex_description;
	tmp_obj2->ex_description = ed;

	CREATE(ed->keyword, char, strlen(tmp_obj2->name) + 1);
	strcpy(ed->keyword, tmp_obj2->name);
	ed->description = strdup(buf);

	tmp_obj2->obj_flags.value[0] = level/5;
	tmp_obj2->obj_flags.value[1] = IN_ROOM(ch);

	obj_to_room(tmp_obj2, IN_ROOM(tmp_ch));

	act("$p suddenly appears.", TRUE, tmp_ch, tmp_obj2, 0, TO_ROOM);
	act("$p suddenly appears.", TRUE, tmp_ch, tmp_obj2, 0, TO_CHAR);

}


ASPELL(spell_control_weather)
{
	if (!ch)
		return;
	
	if (!OUTSIDE(ch)) {
		send_to_char(ch, "You must be outside to influence the weather!\r\n");
		return;
	}

	if (!tar_str || !*tar_str) {
		send_to_char(ch, "Do WHAT to the weather?!  <better> or <worse> "
						 "please.\r\n");
		return;
	}

	if (!stricmp(tar_str, "better")) 
		force_weather_change(24, IN_ZONE(ch));
		
	else if (!stricmp(tar_str, "worse"))
		force_weather_change(-24, IN_ZONE(ch));

	else
		send_to_char(ch, "Do WHAT to the weather?!  <better> or <worse> "
						 "please.\r\n");
} 


ASPELL(spell_ventriloquism)
{
/*
  Blah - tar_str is just the name of the victim, not the text to be
  ventriloquated.  Hmmmm... 

  char buf[MAX_STRING_LENGTH];
	
  if (!ch || !victim)
  return;

  if (!tar_str || !*tar_str) {
  send_to_char(ch, "Ventriloquate what?!\r\n");
  return;
  }

  snprintf(buf, sizeof(buf), "$N says, '%s'", tar_str);
  act(buf, FALSE, ch, 0, victim, TO_ROOM);
*/
}


ASPELL(spell_disintegrate)
{
   struct obj_data *foolz_objs = NULL;
   int save, i;

   if (ch == NULL)
		return;

   if (obj) {
		switch (GET_OBJ_TYPE(obj)) {
			case ITEM_LIGHT:
				save = 19;
				break;
			case ITEM_SCROLL:
				save = 20;
				break;
			case ITEM_STAFF:
			case ITEM_WAND:
				save = 19;
				break;
			case ITEM_WEAPON:
				save = 18;
				break;
			case ITEM_MISSILE:
				save = 20;
				break;
			case ITEM_ARMOR:
				save = 16;
				break;
			case ITEM_WORN:
				save = 18;
				break;
			default:
				save = 19;
				break;
		}
		/*  Save modified by affect on weapons..this is kinda based on
		 *  +5 or so being high  */
		if (GET_OBJ_EXTRA(obj).ITEM_MAGIC && 
			 GET_OBJ_TYPE(obj) == ITEM_WEAPON) {
			for (i = 0; i < MAX_OBJ_AFFECT; i++) {
				if (obj->affected[i].location == APPLY_DAMROLL)
					save -= obj->affected[i].modifier;
			}
		}

		/* A bonus for ac affecting items also */
		for (i = 0; i < MAX_OBJ_AFFECT; i++) {
			if (obj->affected[i].location == APPLY_AC)
				save -= obj->affected[i].modifier / 10;
		}

		if (rand_number(1, 20) < save) {
			act("$n disintegrates $p.", FALSE, ch, obj, 0, TO_NOTVICT);
			act("You disintegrate $p.", FALSE, ch, obj, 0, TO_CHAR);
			extract_obj(obj);
			return;
		} else {
			act("You fail to disintegrate $p.", FALSE, ch, obj, 0, TO_CHAR);
			return;
		}
   }

   if (victim) {
		if (GET_LEVEL(victim) >= LVL_IMMORT) {
			send_to_char(ch, "Nice try..\r\n");
			return;
		}

		/* Note this is extremely powerful, therefore im giving it 2 saves */
		if (mag_savingthrow(victim, SAVING_SPELL, 0)) {
			act("You resist $n's attempt to disintegrate you.", 
				 FALSE, ch, 0, 0, TO_VICT);
			if (IS_NPC(victim))
				hit(victim, ch, TYPE_UNDEFINED);

			return;
		}

		if (mag_savingthrow(victim, SAVING_SPELL, 0)) {
			act("You resist $n's attempt to disintegrate you.", 
				 FALSE, ch, 0, 0, TO_VICT);
			if (IS_NPC(victim))
            hit(victim, ch, TYPE_UNDEFINED);
			return;
		}

		if (victim->desc) {
			close_socket(victim->desc);
			victim->desc = NULL;
		}

		/* Note this is for disintegrating all items the vict carries also..
		 * may wish to comment this off if you feel it too powerful */
		for (i = 0; i < NUM_WEARS; i++)
			if (GET_EQ(victim, i))
				unequip_char(victim, i);

		while (victim->carrying) {
			foolz_objs = victim->carrying;
			extract_obj(foolz_objs);
		}

		extract_char(victim);
		act("$n disintegrates $N!", FALSE, ch, 0, victim, TO_NOTVICT);
		act("You disintegrate $N!", FALSE, ch, 0, victim, TO_CHAR);

		return;
	}
}


ASPELL(spell_recharge)
{
	int restored_charges = 0, explode = 0;
	
	if (ch == NULL || obj == NULL)
		return;

	/* This is on my mud, comment off on yours */
	if (OBJ_FLAGGED(obj, ITEM_NO_RECHARGE)) {
		send_to_char(ch, "This item cannot be recharged.\r\n");
		return;
	}

	if (GET_OBJ_TYPE(obj) == ITEM_WAND) {

		if (GET_OBJ_VAL(obj, 2) < GET_OBJ_VAL(obj, 1)) {
			send_to_char(ch, "You attempt to recharge the wand.\r\n");
			restored_charges = rand_number(1, 5);
			GET_OBJ_VAL(obj, 2) += restored_charges;

			if (GET_OBJ_VAL(obj, 2) > GET_OBJ_VAL(obj, 1)) {
				send_to_char(ch, "The wand is overcharged and explodes!\r\n");
				act("$n overcharges $p and it explodes!", 
					 FALSE, ch, obj, 0, TO_NOTVICT);

				explode = dice(GET_OBJ_VAL(obj, 2), 3);
				if (GET_LEVEL(ch) < LVL_IMMORT)
					GET_HIT(ch) -= explode;
				update_pos(ch);
				extract_obj(obj);
				return;
			} else {
				send_to_char(ch, "You restore %d charges to the wand.\r\n", 
								 restored_charges);
				return;
			}
		} else {
			send_to_char(ch, "That item is already at full charges!\r\n");
			return;
		}
	} else if (GET_OBJ_TYPE(obj) == ITEM_STAFF) {

		if (GET_OBJ_VAL(obj, 2) < GET_OBJ_VAL(obj, 1)) {
			send_to_char(ch, "You attempt to recharge the staff.\r\n");
			restored_charges = rand_number(1, 3);

			GET_OBJ_VAL(obj, 2) += restored_charges;
			if (GET_OBJ_VAL(obj, 2) > GET_OBJ_VAL(obj, 1)) {
				send_to_char(ch, "The staff is overcharged and explodes!\r\n");
				act("$n overcharges $p and it explodes!", 
					 FALSE, ch, obj, 0, TO_NOTVICT);

				explode = dice(GET_OBJ_VAL(obj, 2), 6);
				if (GET_LEVEL(ch) < LVL_IMMORT)
					GET_HIT(ch) -= explode;
				update_pos(ch);
				extract_obj(obj);
				return;
			} else {
				send_to_char(ch, "You restore %d charges to the staff.\r\n", 
								 restored_charges);
				return;

			}
		} else {
			send_to_char(ch, "That item is already at full charges!\r\n");
			return;
		}
   }
}



ASPELL(spell_scry)
{
	struct char_data *i;
	char name[MAX_INPUT_LENGTH];
	int j;

	strlcpy(name, tar_str, sizeof(name));
	j = level >> 1;

	for (i = character_list; i && j > 0; i = i->next) {
		if (!isname(name, i->player.name))
			continue;

		if (IN_ROOM(i) != NOWHERE)
			send_to_char(ch, "%s is in %s.\r\n",
							 CAP(GET_NAME(i)), world[IN_ROOM(i)].name);
		else
			send_to_char(ch, "%s's location is uncertain.\r\n", CAP(GET_NAME(i)));

		j -= 1;
	}

	if (j == level >> 1)
		send_to_char(ch, "You sense nothing.\n\r");
}

