/* ************************************************************************
 *   File: weather.c                                     Part of CircleMUD *
 *  Usage: functions handling time and the weather                         *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "screen.h"


/* extern functions */
ACMD(do_weather);
extern int in_water(struct char_data *ch);
extern void die(struct char_data *ch, struct char_data *killer);

/* extern vars */
extern struct time_info_data time_info;
extern zone_rnum top_of_zone_table;
extern struct zone_data *zone_table;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;

/* local functions */
void weather_and_time(int mode);
void another_hour(int mode);
void weather_change(zone_rnum z);
void force_weather_change(int change, zone_rnum z);
void pressure_change(zone_rnum i);
void sun_text(void);


void weather_and_time(int mode)
{
	zone_rnum i = 0;

	another_hour(mode);
	if (mode)
	{
		for (i = 0; i < top_of_zone_table; i++)
			weather_change(i);
	}
}


int is_night(void) 
{
	if (time_info.hours >= 5 && time_info.hours < 21)
		return (FALSE);
	
	return (TRUE);
}


void sun_text(void)
{
	switch (time_info.hours) {
		case 5:
			sunlight = SUN_RISE;
			send_to_outdoor("The sun rises in the east.\r\n");
			break;
		case 6:
			sunlight = SUN_LIGHT;
			send_to_outdoor("The day has begun.\r\n");
			break;
		case 21:
			sunlight = SUN_SET;
			send_to_outdoor("The sun slowly disappears in the west.\r\n");
			break;
		case 22:
			sunlight = SUN_DARK;
			send_to_outdoor("The night has begun.\r\n");
			break;
		default:
			break;
	}
}



void another_hour(int mode)
{
	time_info.hours++;

	if (mode)
		sun_text();

	if (time_info.hours > 23) {	/* Changed by HHS due to bug ??? */
		time_info.hours -= 24;
		time_info.day++;

		if (time_info.day > 34) {
			time_info.day = 0;
			time_info.month++;

			if (time_info.month > 16) {
				time_info.month = 0;
				time_info.year++;
			}
		}
	}
}


void force_weather_change(int change, zone_rnum z)
{
	zone_table[z].change += change;
	pressure_change(z);
}


void weather_change(zone_rnum z)
{
	int diff;

	if ((time_info.month >= 9) && (time_info.month <= 16))
		diff = (zone_table[z].pressure > 985 ? -2 : 2);
	else
		diff = (zone_table[z].pressure > 1015 ? -2 : 2);

	zone_table[z].change += (dice(1, 4) * diff + dice(2, 6) - dice(2, 6));
	zone_table[z].change = MIN(zone_table[z].change, 12);
	zone_table[z].change = MAX(zone_table[z].change, -12);

	pressure_change(z);
}


void pressure_change(zone_rnum i)
{
	char buf[MAX_STRING_LENGTH];
	int change = 0;
	struct descriptor_data *j;

	zone_table[i].pressure += zone_table[i].change;
	zone_table[i].pressure = MIN(zone_table[i].pressure, 1040);
	zone_table[i].pressure = MAX(zone_table[i].pressure, 960);

	switch (zone_table[i].sky) {
		case SKY_CLOUDLESS:
			if (zone_table[i].pressure < 990)
				change = 1;
			else if (zone_table[i].pressure < 1010)
				if (dice(1, 4) == 1)
					change = 1;
			break;
		case SKY_CLOUDY:
			if (zone_table[i].pressure < 970)
				change = 2;
			else if (zone_table[i].pressure < 990)
				if (dice(1, 4) == 1)
					change = 2;
				else
					change = 0;
			else if (zone_table[i].pressure > 1030)
				if (dice(1, 4) == 1)
					change = 3;

			break;
		case SKY_RAINING:
			if (zone_table[i].pressure < 970)
				if (dice(1, 4) == 1)
					change = 4;
				else
					change = 0;
			else if (zone_table[i].pressure > 1030)
				change = 5;
			else if (zone_table[i].pressure > 1010)
				if (dice(1, 4) == 1)
					change = 5;

			break;
		case SKY_LIGHTNING:
			if (zone_table[i].pressure > 1010)
				change = 6;
			else if (zone_table[i].pressure > 990)
				if (dice(1, 4) == 1)
					change = 6;

			break;
		default:
			change = 0;
			zone_table[i].sky = SKY_CLOUDLESS;
			break;
	}

	switch (change) {
		case 0:
			if (zone_table[i].sky == SKY_RAINING)
				strlcpy(buf, "The rain continues to pour down.\r\n", sizeof(buf));
			else if (zone_table[i].sky == SKY_LIGHTNING)
				strlcpy(buf, "The continuing thunderstorm soaks you to the "
						  "bone.\r\n", sizeof(buf));
			else if (zone_table[i].sky == SKY_CLOUDY)
				strlcpy(buf, "Clouds continue to gather overhead, making the "
						  "sky ominously overcast.\r\n", sizeof(buf));
			else if (zone_table[i].sky == SKY_CLOUDLESS) {
				if (!is_night())
					strlcpy(buf, "The sun shines brightly overhead.\r\n",
							  sizeof(buf));
				else
					strlcpy(buf, "The stars twinkle brightly in the night "
							  "sky.\r\n", sizeof(buf));
			}
			
			break;
		case 1:
			strlcpy(buf, "The sky starts to get cloudy.\r\n", sizeof(buf));
			zone_table[i].sky = SKY_CLOUDY;
			break;
		case 2:
			strlcpy(buf, "It starts to rain.\r\n", sizeof(buf));
			zone_table[i].sky = SKY_RAINING;
			break;
		case 3:
			strlcpy(buf, "The clouds disappear.\r\n", sizeof(buf));
			zone_table[i].sky = SKY_CLOUDLESS;
			break;
		case 4:
			strlcpy(buf, "Lightning starts to show in the sky.\r\n", sizeof(buf));
			zone_table[i].sky = SKY_LIGHTNING;
			break;
		case 5:
			strlcpy(buf, "The rain stops.\r\n", sizeof(buf));
			zone_table[i].sky = SKY_CLOUDY;
			break;
		case 6:
			strlcpy(buf, "The lightning stops.\r\n", sizeof(buf));
			zone_table[i].sky = SKY_RAINING;
			break;
		default:
			break;
	}

	/* We can't just use send_to_outdoor() anymore because 
	 * weather is zone dependent. */
	if (change >= 0 && change <= 6) {
		for (j = descriptor_list; j; j = j->next)
			if (j->connected == CON_PLAYING &&
				 j->character && 
				 AWAKE(j->character) &&
				 OUTSIDE(j->character) && 
				 (IN_ZONE(j->character) == i))
				write_to_output(j, buf);
	}
}


/* ported from Eltanin by AxL, 2feb97 */
/*  added by Miikka M. Kangas 8-14-91 (note this is called with own
 *  pulses!) 
 *
 *  I was getting bored so I wanted to add some edge to weather.
 */
void beware_lightning(void)
{
	int dam = 0;
	zone_rnum i;
	struct char_data *victim = NULL, *temp = NULL;

	for (i = 0; i < top_of_zone_table; i++)
	{
		if (zone_table[i].sky != SKY_LIGHTNING)
			continue;

		if (!rand_number(0, 9)) /* only 10% of the time */
			continue; 

		if (!rand_number(0, 99)) { /* nobody targeted 99% */
			send_to_zone(i, "You hear the clap of distant thunder.\r\n");
			continue;
		}

		for (victim = character_list; victim; victim = temp) {
			temp = victim->next;

			if (OUTSIDE(victim) && IN_ZONE(victim) == i) {
				if  (!rand_number(0, 9)) {  /* hit 10%  */
					dam = rand_number(1, (GET_MAX_HIT(victim) / 2));

					if (GET_LEVEL(victim) >= LVL_IMMORT)
						/* You can't damage an immortal! */
						dam = 0;

					GET_HIT(victim) -= dam;
					act("KAZAK! a lightning bolt hits $n.  You hear a sickening "
						 "sizzle.", TRUE, victim, 0, 0, TO_ROOM);
					act("KAZAK! a lightning bolt hits you.  You hear a sickening "
						 "sizzle.", FALSE, victim, 0, 0, TO_CHAR);

					if (dam > (GET_MAX_HIT(victim) >> 2))
						act("That Really did HURT!", FALSE, victim, 0, 0, TO_CHAR);

					send_to_zone(i, "*BOOM*  You hear the clap of thunder.\r\n");
					update_pos(victim);
					pos_text(NULL, victim, dam);
					
					if (GET_POS(victim) == POS_DEAD) {
						log("Thunderstorm killed %s", GET_NAME(victim));
						die(victim, NULL);
					} 
					break;
				} else { /* number(0,10) */
					act("KAZAK! a lightning bolt hits nearby.",
						 FALSE, victim, 0, 0, TO_ROOM);
					send_to_char(victim, "KAZAK! a lightning bolt hits nearby.\r\n");
					send_to_zone(i, "*BOOM*  You hear the clap of thunder.\r\n");
					break;    /*  only 1 bolt at a time per zone */
				}
			} /* if ((OUTSIDE(victim) == TRUE) && (!IS_NPC(victim))) { */
		} /* for (victim = character_list; victim; victim = temp) { */
	} /* for (i = 0; i < top_of_zone_table; i++) */
}


#define MAP_WIDTH   72
#define SHOW_WIDTH  MAP_WIDTH/2
#define MAP_HEIGHT  9

const char * star_map[] =
{
	"   W.N     ' .     :. M,N     :  y:.,N    `  ,       B,N      .      .  ",
	" W. :.N .      G,N  :M.: .N  :` y.N    .      :     B:   .N       :     ",
	"    W:N    G.N:       M:.,N:.:   y`N      ,    c.N           .:    `    ",
	"   W.`:N       '. G.N  `  : ::.      y.N      c'N      B.N R., ,N       ",
	" W:'  `:N .  G. N    `  :    .y.N:.          ,     B.N      :  R:   . .N",
	":' '.   .    G:.N      .'   '   :::.  ,  c.N   :c.N    `        R`.N    ",
	"      :       `        `        :. ::. :     '  :        ,   , R.`:N    ",
	"  ,       G:.N              `y.N :. ::.c`N      c`.N   '        `      .",
	"     ..        G.:N :           .:   c.N:.    .              .          "
};

/***************************CONSTELLATIONS*******************************
  Lupus     Gigas      Pyx      Enigma   Centaurus    Terken    Raptus
   The       The       The       The       The         The       The  
White Wolf  Giant     Pixie     Sphinx    Centaur      Drow     Raptor
*************************************************************************/	
const char *sun_map[] =
{
	")'|'(",
	"- O -",
	").|.("
}; 

const char *moon_map[] =
{
	" @@@ ",
	"@@@@@",
	" @@@ "
};


void look_at_sky(struct char_data *ch)
{
	static char buf[MAX_STRING_LENGTH];
	static char buf2[4];
	int starpos, sunpos, moonpos, moonphase, i, linenum;

   if (zone_table[IN_ZONE(ch)].sky > 0 && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "The cloudy sky obscures your vision.\r\n");
		do_weather(ch, 0, 0, 0);
		return;
   }

   send_to_char(ch, "You gaze up towards the heavens and see:\r\n");

   sunpos  = (MAP_WIDTH * (24 - time_info.hours) / 24);
   moonpos = (sunpos + time_info.day * MAP_WIDTH / NUM_DAYS) % MAP_WIDTH;
   if ((moonphase = ((((MAP_WIDTH + moonpos - sunpos) % MAP_WIDTH) +
                      (MAP_WIDTH/16)) * 8) / MAP_WIDTH) > 4) moonphase -= 8;
   starpos = (sunpos + MAP_WIDTH * time_info.month / NUM_MONTHS) % MAP_WIDTH;
   /* The left end of the star_map will be straight overhead at
	 * midnight during month 0 */

   for (linenum = 0; linenum < MAP_HEIGHT; linenum++) {
		if ((time_info.hours >= 6 && time_info.hours <= 18) &&
			 (linenum < 3 || linenum >= 6))
			continue;

		strlcpy(buf, QWHT, sizeof(buf));

		for (i = MAP_WIDTH/4; i <= 3*MAP_WIDTH/4; i++) {
			/* plot moon on top of anything else...unless new moon & no eclipse */
			if ((time_info.hours >= 6 && time_info.hours <= 18) &&  /* daytime? */
				 (moonpos >= MAP_WIDTH/4 - 2) && (moonpos <= 3*MAP_WIDTH/4 + 2) && /* in sky? */
				 (i >= moonpos - 2) && (i <= moonpos + 2) && /* is this pixel near moon? */
				 ((sunpos == moonpos && time_info.hours == 12) || moonphase != 0) && /*no eclipse*/
				 (moon_map[linenum-3][i+2-moonpos] == '@')) {
				if ((moonphase < 0 && i - 2 - moonpos >= moonphase) ||
					 (moonphase > 0 && i + 2 - moonpos <= moonphase)) {
					strcat(buf, QWHT);
					strcat(buf, "@");
				} else
					strcat(buf, " ");
			} else if ((linenum >= 3) && (linenum < 6) && /* nighttime */
						  (moonpos >= MAP_WIDTH/4 - 2) && (moonpos <= 3*MAP_WIDTH/4 + 2) && /* in sky? */
						  (i >= moonpos - 2) && (i <= moonpos + 2) && /* is this pixel near moon? */
						  (moon_map[linenum-3][i+2-moonpos] == '@')) {
				if ((moonphase < 0 && i - 2 - moonpos >= moonphase) ||
					 (moonphase > 0 && i + 2 - moonpos <= moonphase)) {
					strcat(buf, QWHT);
					strcat(buf, "@");
				} else
					strcat(buf, " ");
			} else { /* plot sun or stars */
				if (time_info.hours >= 6 && time_info.hours <= 18) { /* daytime */
					if (i >= sunpos - 2 && i <= sunpos + 2) {
						snprintf(buf2, sizeof(buf2), "%s%c",
									QYEL, sun_map[linenum-3][i+2-sunpos]);
						strcat(buf, buf2);
					} else
						strcat(buf, " ");
				} else {
					switch (star_map[linenum][(MAP_WIDTH + i - starpos)%MAP_WIDTH]){
						default: strcat(buf, " ");    break;
						case '.': strcat(buf, ".");   break;
						case ',': strcat(buf, ",");   break;
						case ':': strcat(buf, ":");   break;
						case '`': strcat(buf, "`");   break;
						case 'R': strcat(buf, QRED);  break;
						case 'G': strcat(buf, QGRN);  break;
						case 'B': strcat(buf, QBLU);  break;
						case 'W': strcat(buf, QWHT);  break;
						case 'M': strcat(buf, QMAG);  break;
						case 'N': strcat(buf, QNRM);  break;
						case 'y': strcat(buf, QYEL);  break;
						case 'c': strcat(buf, QCYN);  break;
					}
				}
			}
		}
		send_to_char(ch, "%s%s\r\n", buf, QNRM);
   }
}


void check_fishing(void)
{
	struct descriptor_data *d;
	struct char_data *ch;
	int bite;

	for (d = descriptor_list; d; d = d->next) {
		if (d->connected)
			continue;

		ch = d->character;
		
		if (PLR_FLAGGED(ch, PLR_FISHING) && !in_water(ch))
			 PLR_FLAGS(ch).PLR_FISHING = 0;

		if (PLR_FLAGGED(ch, PLR_FISH_ON)) {
			send_to_char(ch, "The fish continues to struggle on your line!\r\n");
			continue;
		}
		
		if (PLR_FLAGGED(ch, PLR_FISHING)) {
			bite = rand_number(1, 10);

			/* Better luck in the rain */
			if (zone_table[IN_ZONE(ch)].sky == SKY_RAINING ||
				 zone_table[IN_ZONE(ch)].sky == SKY_LIGHTNING)
				bite -= 1;

			/* And at night, especially at dusk and twilight */
			if (sunlight == SUN_RISE || sunlight == SUN_SET)
				bite -= 2;
			else if (sunlight == SUN_DARK)
				bite -= 1;

			/* Immorts have a slightly better chance :) */
			if (GET_LEVEL(ch) > LVL_IMMORT)
				bite -= (GET_LEVEL(ch) - LVL_IMMORT);

			if (bite >= 7 && bite <= 8) {
				send_to_char(ch, "Time goes by... not even a nibble.\r\n");
			} else if (bite >= 6) {
				send_to_char(ch, "You feel a slight jiggle on your line.\r\n");
			} else if (bite >= 4) {
				send_to_char(ch, "You feel a very solid pull on your line!\r\n");
				PLR_FLAGS(ch).PLR_FISH_ON = 1;
			} else if (bite >= 2) {
				send_to_char(ch, "Your line suddenly jumps to life, FISH ON!!!\r\n");
				PLR_FLAGS(ch).PLR_FISH_ON = 1;
			}
		}
	}
}
